// NOImage.h
//
// NOImage: an image
//			
//
// (c) NiXPS NV, March 2007

#ifndef _NOIMAGE_H_
#define _NOIMAGE_H_

/*! \file NOImage.h
    \brief Header file for NOImage class
*/

#include "NTypes.h"
#include "NTImageFormat.h"
#include "NOPartBased.h"

extern "C"
{
	DLLIMPORTEXPORT NOImageHandler NOImage_new(NOHandler* pHandler, NCommon::UInt16 pID);
	DLLIMPORTEXPORT NOImageHandler NOImage_new2(NOPackageHandler *pHandler, NCommon::UInt16 pID);
	DLLIMPORTEXPORT void NOImage_getFileName(NOImageHandler* pImage, NOStringHandler* pString);
	DLLIMPORTEXPORT NCommon::UInt32 NOImage_getFormat(NOImageHandler* pImage);
	DLLIMPORTEXPORT NCommon::UInt32 NOImage_writeImageToFile(NOImageHandler* pImage, NCommon::UTF8Char *destinationPath);
	DLLIMPORTEXPORT NCommon::UInt32 NOImage_getImageData(NOImageHandler* pImage, NCommon::UInt8 *pBuffer, NCommon::UInt32 pSize);
	DLLIMPORTEXPORT NCommon::UInt32 NOImage_getImageSize(NOImageHandler* pImage);
	DLLIMPORTEXPORT NCommon::UInt32 NOImage_getWidthInPixels(NOImageHandler* pImage);
	DLLIMPORTEXPORT NCommon::UInt32 NOImage_getHeightInPixels(NOImageHandler* pImage);
	DLLIMPORTEXPORT double NOImage_getXResolution(NOImageHandler* pImage);
	DLLIMPORTEXPORT double NOImage_getYResolution(NOImageHandler* pImage);
	DLLIMPORTEXPORT NCommon::UInt32 NOImage_getColorType(NOImageHandler* pImage);
	DLLIMPORTEXPORT NCommon::UInt32 NOImage_getBitDepth(NOImageHandler* pImage);
}

#ifdef NIXPS_CPP_INTERFACE
namespace NiXPSObjects
{

class NOPackage;

using namespace NiXPSAccess;

/**
 * \class NOImage NOImage.h "NOImage.h"
 * Class representing an image in an XPS package
 */
class NOImage: public NOPartBased
{
	friend class std::vector<NiXPSObjects::NOImage>;

	public:

		NOImage(NOImageHandler pHandler);

		NOImage(NOPackage *pPackage, NCommon::UInt16 pID);
		
		// destructor
		virtual ~NOImage();

		/// Returns the filename of the image.  By default returns the base name in the XPS package
		const NCommon::UTF8String getFileName();

		/// Returns the image Format
		NCommon::Image::Format getFormat();
		
		/// Write the image to the path destinationPath.  Overwrites if file already exists.
		/// @param destinationPath UTF-8 encoded string describing the path
		///			to write the file to.  The path is seperated by forward slashes ('/').
		void writeImageToFile(const NCommon::UTF8Char *destinationPath);

		/// Fill a buffer with the image data
		/// @param pBuffer The buffer that will contain the image data upon successful completion
		/// @param pSize The size of the buffer.  This operation will fail if the size of the buffer is
		///		smaller than the image size returned by getImageSize()
		/// @return True upon successful completion, false otherwise
		bool getImageData(NCommon::UInt8 *pBuffer, NCommon::UInt32 pSize);
		
		/// Returns the size of the image in bytes (UInt8's)
		NCommon::UInt32 getImageSize() const;

		/// Returns the width of the image in pixels
		NCommon::UInt32 getWidthInPixels() const;

		/// Returns the height of the image in pixels
		NCommon::UInt32 getHeightInPixels() const;
		
		/// Returns resolution of the image in the X direction
		double getXResolution() const;

		/// Returns resolution of the image in the Y direction
		double getYResolution() const;

		/// Returns the colortype of the image
		NCommon::Image::ColorType getColorType() const;
		
		/// Returns the bitdepth of the image
		NCommon::Image::BitDepth getBitDepth() const;
};

}

#endif // NIXPS_CPP_INTERFACE
#endif
