// NOLibrary.h
//
// NOLibrary
//			
// (c) NiXPS NV, January 2008

#ifndef _NOLIBRARY_H_
#define _NOLIBRARY_H_

/*! \file NOLibrary.h
    \brief Header file for NOLibrary functions
*/

#include "NTypes.h"
#include "NTImageFormat.h"
#include "NOException.h"
#include "NOPartBased.h"
#include "NOPackage.h"
#include "NODocument.h"
#include "NOPage.h"
#include "NOCoreProperties.h"
#include "NOFontVector.h"
#include "NOPackageMap.h"
#include "NOUtility.h"
#include "NOOptions.h"
#include "NOXPrefix.h"

extern "C"
{
	DLLIMPORTEXPORT void NOLibrary_Initialize(NCommon::UTF8Char *pLicenseKey);
	DLLIMPORTEXPORT void NOLibrary_GetVersion(NCommon::UInt32 *pMajor, NCommon::UInt32 *pMinor, NCommon::UInt32 *pRevision, NCommon::UInt32 *pBuild, NCommon::UInt32 *pState);
}

#ifdef NIXPS_CPP_INTERFACE
namespace NiXPSObjects
{

/*!
 * \namespace NiXPSObjects::NOLibrary
 * Namespace with functions to initialize library and ask some properties of it
 */
namespace NOLibrary
{
	/// Initializes the library with a license key, this is required to be able to run the library without trial banners
	/// @param pLicenseKey Your personal license key
	void Initialize(const NCommon::UTF8Char *pLicenseKey);

	/// Retrieves version information about the library
	/// @param pMajor The major version number.
	/// @param pMinor The minor version number.
	/// @param pRevision The revision number.
	/// @param pBuild The build number.
	/// @param pState The release state (0:dev, 1:beta, 2:rc, 3:release).
	void GetVersion(NCommon::UInt32 &pMajor, NCommon::UInt32 &pMinor, NCommon::UInt32 &pRevision, NCommon::UInt32 &pBuild, NCommon::UInt32 &pState);
};

}

#endif // NIXPS_CPP_INTERFACE

#endif	// _NOLIBRARY_H_
