// NOPackage.h
//
// Object that represents a complete xps package
//			
//
// (c) NiXPS NV, October 2006

/*! \file NOPackage.h
    \brief Header file for NOPackage class
*/

#ifndef _NOXPSPACKAGE_H_
#define _NOXPSPACKAGE_H_

#include "NTypes.h"
#include "NOProgressReporter.h"
#include "NOPartWithResources.h"
#include "NOFontVector.h"
#include "NOImageVector.h"

// C api
extern "C"
{	
	DLLIMPORTEXPORT NOPackageHandler NOPackage_create(NCommon::UInt32 pCreateEmptyDoc, NCommon::UTF8Char *pName);
	DLLIMPORTEXPORT void NOPackage_destroyPackage(NOPackageHandler* pPackage);
	DLLIMPORTEXPORT NOPackageHandler NOPackage_readPackageFromFile(NCommon::UTF8Char *pFilename);
	DLLIMPORTEXPORT NOPackageHandler NOPackage_createFromImage(NCommon::UTF8Char *pFilename);
	DLLIMPORTEXPORT NOPackageHandler NOPackage_readPackageFromBuffer(NCommon::UTF8Char *pFilename, NCommon::UInt8* pBuffer, NCommon::UInt32 pSize);
	DLLIMPORTEXPORT void NOPackage_writePackageToFile(NOPackageHandler* pPackage, NCommon::UTF8Char *pFilename);
	DLLIMPORTEXPORT NCommon::UInt32 NOPackage_writePackageToBuffer(NOPackageHandler *pDocument, NCommon::UInt8 *pBuffer, NCommon::UInt32 pBufSize, NCommon::UInt32 *pFileSize);
	DLLIMPORTEXPORT void NOPackage_save(NOPackageHandler* pPackage);
	DLLIMPORTEXPORT void NOPackage_getFilename(NOPackageHandler* pPackage, NOStringHandler* pString);
	DLLIMPORTEXPORT void NOPackage_setFilename(NOPackageHandler* pPackage, NCommon::UTF8Char* pString);
	DLLIMPORTEXPORT NCommon::UInt32 NOPackage_hasFilename(NOPackageHandler* pPackage);
	DLLIMPORTEXPORT NCommon::UInt32 NOPackage_getDirty(NOPackageHandler* pPackage);
	DLLIMPORTEXPORT NCommon::UInt32 NOPackage_hasCoreProperties(NOPackageHandler* pPackage);
	DLLIMPORTEXPORT NOCorePropertiesHandler NOPackage_getCoreProperties(NOPackageHandler* pPackage);
	DLLIMPORTEXPORT NOCorePropertiesHandler NOPackage_createCoreProperties(NOPackageHandler* pPackage);
	DLLIMPORTEXPORT NCommon::UInt32 NOPackage_getNumberOfDocuments(NOPackageHandler* pPackage);
	DLLIMPORTEXPORT NODocumentHandler NOPackage_getDocument(NOPackageHandler* pPackage, NCommon::UInt32 pNum);
	DLLIMPORTEXPORT void NOPackage_addDocument(NOPackageHandler* pPackage, NODocumentHandler pDoc, NOProgressReporterHandler pReporter);
	DLLIMPORTEXPORT NCommon::UInt32 NOPackage_getDocumentNumberOf(NOPackageHandler* pPackage, NODocumentHandler pDoc);
	DLLIMPORTEXPORT NCommon::UInt32 NOPackage_hasThumbnail(NOPackageHandler* pPackage);
	DLLIMPORTEXPORT NOThumbnailHandler NOPackage_getThumbnail(NOPackageHandler* pPackage);
	DLLIMPORTEXPORT NCommon::UInt32 NOPackage_getNumberOfPages(NOPackageHandler* pPackage);
	DLLIMPORTEXPORT void NOPackage_setDirty(NOPackageHandler *pPackage, NCommon::UInt32 pDirtyState);
	DLLIMPORTEXPORT NCommon::UInt32 NOPackage_getFonts(NOPackageHandler *pPackage, NOFontVectorHandler pFontsHandler);
	DLLIMPORTEXPORT NCommon::UInt32 NOPackage_getImages(NOPackageHandler *pPackage, NOImageVectorHandler pImagesHandler);
	DLLIMPORTEXPORT NCommon::UInt32 NOPackage_getParts(NOPackageHandler *pPackage, NOVectorHandler pPartsHandler);
	DLLIMPORTEXPORT NCommon::UInt32 NOPackage_getTypeForID(NOPackageHandler *pPackage, NCommon::UInt32 pID);
	DLLIMPORTEXPORT NCommon::UInt32 NOPackage_exportToPDF(NOPackageHandler *pDocument, NCommon::UTF8Char *pPDFFileName, NOProgressReporterHandler lProgressReporter);
	DLLIMPORTEXPORT NCommon::UInt32 NOPackage_exportToPDFWithOptions(NOPackageHandler *pDocument, NCommon::UTF8Char *pPDFFileName, NOProgressReporterHandler lProgressReporter, NOOptionsHandler pOptions);
	DLLIMPORTEXPORT NCommon::UInt32 NOPackage_exportToPDFWithOptionsInMemory(NOPackageHandler *pDocument, NCommon::UInt8 *pBuffer, NCommon::UInt32 pBufSize, NCommon::UInt32 *pPDFSize, NOProgressReporterHandler lProgressReporter, NOOptionsHandler pOptions);
}

#ifdef NIXPS_CPP_INTERFACE
// C++ api
namespace NiXPSObjects
{

class NODocument;
class NOCoreProperties;
class NODocument;
class NOFontVector;
class NOImageVector;
class NOOptions;

using namespace NiXPSAccess;

/*!
 * \class NOPackage NOPackage.h "NOPackage.h"
 * Use an NOPackage to read an XPS-file from disk and access or manipulate the content of the package.
 */
class NOPackage: public NOPartWithResources
{
	public:		
		// constructor
		// NOPackage(NAPackage &pPackage, const UInt16 &pID);
		NOPackage(NOPackageHandler pHandler);

		// destructor
		virtual ~NOPackage();

		/// @name Constructing and destructing an NOPackage
		//@{

		/// Reads a package from a file.
		/// @param pFilename Full path to the XPS-file to read.
		static NOPackage * readPackageFromFile(const UTF8Char *pFilename);
		
		/// Create an XPS package from an image
		/// @param pFilename Full path to the image to create the file from
		static NOPackage * createFromImage(const UTF8Char *pFilename);

		/// Reads a package from a file.
		/// @param pFilename The name of the XPS-file.
		/// @param pBuffer Memory buffer holding the XPS file. The library does not take ownership, so make sure the buffer is kept alive.
		/// @param pSize Size of the file in memory.
		static NOPackage * readPackageFromBuffer(const UTF8Char *pFilename, const UInt8* pBuffer, const UInt32 pSize);

		/// Creates an empty package.
		/// @param pCreateEmptyDoc Create an empty document in the package
		static NOPackage * createPackage(bool pCreateEmptyDoc, const UTF8Char *pName=0);
		
		/// Destroy a package.  You need to call this to dispose of a package and all the resources it keeps.
		/// @param pXPSPackage Package to destroy
		static void destroyPackage(NOPackage *pXPSPackage);

		//@}
				
		/// Writes the package to a file
		/// @param pFilename Full path to the XPS-file to write.
		void writePackageToFile(const UTF8Char *pFilename);

		/// Writes the package to a file
		void writePackageToFile();

		/// Writes the package to a buffer
		UInt32 writePackageToBuffer(UInt8 *pBuffer, const UInt32 pBufSize, UInt32 &pFileSize);

		// returns true if we have a filename
		bool hasOutputFilename() const;

		// returns the filename
		const UTF8String getOutputFilename() const;
		
		// sets the output filename
		void setOutputFilename(const UTF8Char *pFilename);

		/// Exports the package to a PDF file.  All pages from all documents will be put into one document
		/// @param pPDFFileName the path of the file to write
		/// @param pProgressReporter This progress reporter object gets polled regularly with % update of the export process
		UInt32 exportToPDF(const UTF8Char *pPDFFileName, NOProgressReporter &pProgressReporter);
		
		/// Exports the package to a PDF file.  All pages from all documents will be put into one document
		/// @param pPDFFileName the path of the file to write
		/// @param pProgressReporter This progress reporter object gets polled regularly with % update of the export process
		/// @param pOptions These are the options. Currently the following opsions are supported:
		///
		///	"Author": The Author that gets stored in the PDF metadata
		///	"Title": The Title that gets stored in the PDF metadata
		///	"Subject": The Subject that gets stored in the PDF metadata
		///	"Keywords": The Keywords that gets stored in the PDF metadata
		///	"Creator": The Creator that gets stored in the PDF metadata
		///	"Producer": The Producer that gets stored in the PDF metadata
		///	"PDF Conformance": PDF conformance level, only valid value is "PDF/A-1B" for PDF/A output
		///	"Base14FontEmbedding": "Embed"(default) or "DoNotEmbed" the base 14 font files
		///	"Base14SimilarFonts": "Embed"(default) or "DoNotEmbed" fonts that look similar to base 14 fonts (if not embedded, they are replaced by their similar base 14 counterpart)
		///	"CropImages": "false"(default) or "true"; if the image is bigger than the box used on the page, this option will crop the image data
		///	"Outline": "No"(default) or "Yes"; include the XPS document outline in the PDF
		UInt32 exportToPDFWithOptions(const UTF8Char *pPDFFileName, NOProgressReporter &pProgressReporter, NOOptions &pOptions);

		/// Exports the package to a PDF file.  All pages from all documents will be put into one document.
		/// This version saves the resulting PDF to a memory buffer
		UInt32 NOPackage::exportToPDFWithOptions(UInt8 *pBuffer, const UInt32 pBufSize, UInt32 &pPDFSize, NOProgressReporter &pProgressReporter, NOOptions &pOptions);

		/// Returns whether the package is dirty
		bool getDirty();
		
		/// Set the package dirty state
		void setDirty(bool pDirty);

		/// Returns whether we have a core properties object
		bool hasCoreProperties();

		/// Returns the NOCoreProperties object, an object represinting the Core Properties of the XPS Package.
		NOCoreProperties getCoreProperties();

		/// @name Manipulating documents in a package
		//@{ 
		// == Document related members
		
		/// Returns the number of documents in this package
		NCommon::UInt32 getNumberOfDocuments();

		/// Returns the document number of a given NODocument in this xps package
		/// @param pDoc The NODocument that the library needs to find the Document Number of.
		NCommon::UInt32 getDocumentNumberOf(const NODocument &pDoc);

		/// Returns the NODocument at postion pNum.
		/// @param pNum The document position
		NODocument getDocument(const NCommon::UInt32 pNum);

		/// Add a document to the package
		void addDocument(const NODocument &pDoc, NOProgressReporter &pReporter);

		/// Returns whether we have a thumbnail
		virtual bool hasThumbnail();

		/// Returns the thumbnail represented by a NOThumbnail object
		NOThumbnail getThumbnail();

		/// Returns the total number of pages in all documents
		NCommon::UInt32 getNumberOfPages();
		
		/// Returns the total number of fonts in all documents and a font vector containing all those fonts
		NCommon::UInt32 getFonts(NOFontVector &pFonts);

		/// Returns the total number of images in all documents and an image vector containing all those images
		NCommon::UInt32 getImages(NOImageVector &pImages);

		/// Returns the total number of images in all documents and an image vector containing all those images
		NCommon::UInt32 getParts(NOVector &pParts);
};

}
#endif // NIXPS_CPP_INTERFACE
#endif
