// NOPackageMap.h
//
// Object that represents a map of xps packages
//			
// (c) NiXPS NV, February 2008

/*! \file NOPackageMap.h
    \brief Header file for NOPackageMap class
*/

#ifndef _NOPACKAGEMAP_H_
#define _NOPACKAGEMAP_H_

#include "NTypes.h"

// C api
extern "C"
{
	DLLIMPORTEXPORT NOPackageMapHandler NOPackageMap_new();
	DLLIMPORTEXPORT void NOPackageMap_delete(NOPackageMapHandler *pHandler);
	DLLIMPORTEXPORT void NOPackageMap_addEntry(NOPackageMapHandler *pHandler, NCommon::UTF8Char *pString, NOPackageHandler pPackage);
	DLLIMPORTEXPORT NCommon::UInt32 NOPackageMap_size(NOPackageMapHandler *pHandler);
	DLLIMPORTEXPORT void NOPackageMap_getEntry(		NOPackageMapHandler *pHandler,
													NCommon::UInt32 pIndex,
													NOStringHandler *pStringHandler,
													NOPackageHandler *pPackageHandler);
	DLLIMPORTEXPORT void NOPackageMap_clear(	NOPackageMapHandler *pHandler);
};

#ifdef NIXPS_CPP_INTERFACE
// C++ api

#include <map>

namespace NiXPSObjects
{

class NOPackage;

using namespace NiXPSAccess;

/*!
 * \class NOPackageMap NOPackageMap.h "NOPackageMap.h"
 * NOPackageMap is the class used to manage a map of packages
 */
class NOPackageMap
{
	// declare don't define ... these shouldn't be used
	NOPackageMap();
	NOPackageMap(const NOPackageMap &pArgument);
	NOPackageMap &operator=(const NOPackageMap &pArgument);
	
	public:
		NOPackageMapHandler mHandler;

		NOPackageMap(NOPackageMapHandler pHandler);

		// destructor
		virtual ~NOPackageMap();

		/// create a package from a map of string - NOPackage pointers
		/// Useful if you want to run an XML-script using NOUtility::runScript.  What you put
		/// in the map depends on the XML-script you are using
		/// @param pMap The std::map to construct the NOPackageMap with
		NOPackageMap(const std::map<NCommon::UTF8String, NiXPSObjects::NOPackage *> &pMap);
		
		/// get a C-safe struct from this NOPackageMap to handle over to the DLL
		NOPackageMapHandler getHandler() const { return mHandler; }
};

}

#endif // NIXPS_CPP_INTERFACE

#endif // _NOPACKAGEMAP_H_
