// NOPage.h
//
// NOPage: NOPage class
//			
//
// (c) NiXPS NV, November 2006

/*! \file NOPage.h
    \brief Header file for NOPage class
*/

#ifndef _NOPAGE_H_
#define _NOPAGE_H_

#include "NTypes.h"
#include "NOPartWithResources.h"
#include "NOThumbnail.h"
#include "NODocument.h"
#include "NOFont.h"
#include "NOImage.h"
#include "NOFontVector.h"
#include "NOImageVector.h"
#include "NOXPrefix.h"
#include "NOTextSelection.h"
#include "NOSelection.h"

// C api
extern "C"
{
	DLLIMPORTEXPORT NOPageHandler NOPage_new(NOHandler* pHandler, NCommon::UInt16 pID);
	DLLIMPORTEXPORT NODocumentHandler NOPage_getDocument(NOPageHandler* pPage);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_getFonts(NOPageHandler* pPage, NOVectorHandler pFonts);
	DLLIMPORTEXPORT NOFontHandler NOPage_getFont(NOPageHandler* pPage, NCommon::UTF8Char *pURI);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_getImages(NOPageHandler* pPage, NOVectorHandler pImages);
	DLLIMPORTEXPORT NOImageHandler NOPage_getImage(NOPageHandler* pPage, NCommon::UTF8Char *pURI);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_getPageNumber(NOPageHandler* pPage);
	DLLIMPORTEXPORT NOFontHandler NOPage_addFont(NOPageHandler* pPage, NOFontHandler pFont);
	DLLIMPORTEXPORT NOFontHandler NOPage_addFontFile(NOPageHandler* pPage, NCommon::UTF8Char *pFontFile);
	DLLIMPORTEXPORT NOImageHandler NOPage_addImage(NOPageHandler* pPage, NOImageHandler pImage);
	DLLIMPORTEXPORT NOImageHandler NOPage_addImageFile(NOPageHandler* pHandler, NCommon::UTF8Char *pImageFile);
	DLLIMPORTEXPORT void NOPage_copyResourcesFrom(NOPageHandler* pPage, NOPageHandler* pFromPage);
	DLLIMPORTEXPORT void NOPage_normalizeContentStream(NOPageHandler* pPage);
	DLLIMPORTEXPORT void NOPage_getPageSize(NOPageHandler* pPage, double *pWidth, double *pHeight);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_renderToImage(NOPageHandler *pPage, unsigned char *pImageBuffer,
							NCommon::UInt32 pWidth, NCommon::UInt32 pHeight, NCommon::UInt32  pUseAntiAliasing, NCommon::UInt32 pOutlineMode);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_renderToImageDirect(NOPageHandler *pPage, unsigned char *pImageBuffer,
							NCommon::UInt32 pWidth, NCommon::UInt32 pHeight);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_renderToImagePartial(NOPageHandler *pPage, unsigned char *pImageBuffer,
							NCommon::UInt32 pWidth, NCommon::UInt32 pHeight, NCommon::doubleRectC pXPSRect, NCommon::UInt32 pAntiAliasing, NCommon::UInt32 pOutlineMode);							
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_renderToTiff(NOPageHandler *pPage, NCommon::UTF8Char *pFile, 
							NCommon::UInt32 pWidth, NCommon::UInt32 pHeight, NCommon::UInt32 pResolutionInPixelsPerInch, NCommon::UInt32 pAntiAliasing);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_renderToGDI(NOPageHandler *pHandler, void *pHDC/*, double pDPI*/);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_exportToSVG(NOPageHandler *pPage, NCommon::UTF8Char *pFile);
	DLLIMPORTEXPORT NOXHandler NOPage_getFixedPage(NOHandler* pHandler);
	DLLIMPORTEXPORT NOXHandler NOPage_createFixedPage(NOHandler* pHandler);
	DLLIMPORTEXPORT void NOPage_checkpoint(NOPageHandler* pPage);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_undo(NOPageHandler* pPage);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_redo(NOPageHandler* pPage);
	DLLIMPORTEXPORT void NOPage_getContent(NOHandler *pHandler, NOStringHandler *pString);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_findObjectAt(NOPageHandler *pPage, NCommon::UInt32 pX, NCommon::UInt32 pY, NCommon::UInt32 pWidth, NCommon::UInt32 pHeight);
	DLLIMPORTEXPORT void NOPage_getObject(NOHandler *pHandler, NCommon::UInt32 pID, NOXHandler *pNOXHandler);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_findAndSelectText(NOPageHandler *pPage, const NCommon::UTF8Char *pSearchString, NCommon::UInt32 pOccurrence);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_findAndSelectTextAll(NOPageHandler *pPage, const NCommon::UTF8Char *pSearchString);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_findText(NOPageHandler *pHandler, const NCommon::UTF8Char *pSearchString, NOVoidHandler pIDs);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_hasTextSelection(NOPageHandler *pPage);
	DLLIMPORTEXPORT NOXHandler NOPage_getStoryFragments(NOHandler* pHandler);
	DLLIMPORTEXPORT void NOPage_clearTextSelection(NOPageHandler *pPage);
	DLLIMPORTEXPORT void NOPage_selectTextMarquee(NOPageHandler *pPage, NCommon::doubleRectC pRectangle);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_getIDForTextSelection(NOPageHandler *pPage);
	DLLIMPORTEXPORT void NOPage_replaceText(NOPageHandler *pPage, NCommon::UInt32 pTextObjectID, const NCommon::UTF8Char *pSearch, const NCommon::UTF8Char *pReplace);
	DLLIMPORTEXPORT NOTextSelectionHandler NOPage_getTextSelection(NOPageHandler *pPage);
	DLLIMPORTEXPORT NCommon::doubleRectC NOPage_getBoundingBox(NOPageHandler *pPage, NCommon::UInt32 pId);
	DLLIMPORTEXPORT void NOPage_moveObject(NOPageHandler *pPage, NCommon::UInt32 pId, double pDeltaX, double pDeltaY);
	DLLIMPORTEXPORT void NOPage_removeObject(NOPageHandler *pPage, NCommon::UInt32 pId);
	DLLIMPORTEXPORT NOSelectionHandler NOPage_getCopyOfSelection(NOPageHandler *pPage);
	DLLIMPORTEXPORT void NOPage_selectObject(NOPageHandler *pPage, NCommon::UInt32 pId, NCommon::UInt32 pIsSelected);
	DLLIMPORTEXPORT void NOPage_clearSelection(NOPageHandler *pPage);
	DLLIMPORTEXPORT void NOPage_swapSelection(NOPageHandler *pPage, NCommon::UInt32 pId);
	DLLIMPORTEXPORT void NOPage_selectAll(NOPageHandler *pPage);
	DLLIMPORTEXPORT void NOPage_selectMarquee(NOPageHandler *pPage, NCommon::UInt32RectC pRect, NCommon::UInt32 pWidth, NCommon::UInt32 pHeight, NCommon::UInt32 pSwap, NCommon::UInt32 pIntersect);
	DLLIMPORTEXPORT void NOPage_selectXPSMarquee(NOPageHandler *pHandler, NCommon::doubleRectC pRect, NCommon::UInt32 pSwap, NCommon::UInt32 pIntersect);
	DLLIMPORTEXPORT NOSelectionHandler NOPage_getObjectsInParagraph(NOPageHandler *pHandler, NCommon::UInt32 pObjectID);
	DLLIMPORTEXPORT void NOPage_getTextProperties(	NOPageHandler *pPage,
													NCommon::UInt32 pTextObjectID,
													double *pX1, double *pY1,
													double *pX2, double *pY2,
													double *pX3, double *pY3,
													double *pX4, double *pY4,
													double *pSize				);
	DLLIMPORTEXPORT NOFontHandler NOPage_addFontWithFamilyAndStyle(NOPageHandler *pHandler, NCommon::UTF8Char *pFamilyName, NCommon::UTF8Char *pStyleName);
	DLLIMPORTEXPORT NOXHandler NOPage_copyPageOnTop(NOPageHandler* pHandler, NOPageHandler pPage);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_isTransparentObject(NOPageHandler *pPage, NCommon::UInt32 pId);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_isRectangularObject(NOPageHandler *pPage, NCommon::UInt32 pId);
	DLLIMPORTEXPORT NCommon::UInt32 NOPage_flattenTransparency(NOPageHandler *pPage);
}

#ifdef NIXPS_CPP_INTERFACE
// C++ api
namespace NiXPSObjects
{

using namespace NCommon;

/*!
 * \class NOPage NOPage.h "NOPage.h"
 * Class representing a page in an XPS package.
 */
class NOPage: public NOPartWithResources
{
	private:
		
	public:
		NOPage();
		
		NOPage(const NOPageHandler pHandler);

		NOPage(NOPackage *pPackage, UInt16 pPageID);

		virtual ~NOPage();

		/// Returns the count of the number of fonts on this page and a vector with all the fonts.
		/// @param pFonts The vector to put the fonts in
		UInt32 getFonts(NOFontVector &pFonts);

		/// Returns the font corresponding to the URI
		/// @param pURI The URI of the font to get as a zero terminated UTF8Char pointer
		NOFont getFont(const UTF8Char *pURI);

		/// Add a font to this page and returns a handler to the new font
		/// @param pFont The font to add
		NOFont addFont(const NOFont &pFont);

		/// Add a font to this page and returns a handler to the new font
		/// @param pFontFile The font file to add as font
		NOFont addFont(const UTF8Char *pFontFile);
		
		/// Add a font with a certain family name and font style to this page
		/// Searches the system fonts.  Returns a font in error state if it doesn't find an appropriate font
		NOFont addFontWithFamilyAndStyle(const UTF8Char *pFontFamilyName, const UTF8Char *pFontStyleName);

		/// Returns the count of the number of images on this page and a vector with all the images
		/// @param pImages The vector to put the images in
		UInt32 getImages(NOImageVector &pImages);
		
		/// Returns the image corresponding to the URI
		/// @param pURI The URI of the image to get as a UTF8String
		NOImage getImage(const UTF8Char *pURI);
		
		/// Add an image to this page and returns a handler to the new image
		/// @param pImage The image to add
		NOImage addImage(const NOImage &pImage);

		/// Add an image to this page and returns a handler to the new image
		/// @param pImage The image file to add
		NOImage addImage(const UTF8Char *pFontFile);

		/// Return the package this page belongs to
		NOPackage getPackage();
		
		/// Get the page size
		/// @param pWidth A pointer to a double to fill in with the width.
		/// @param pHeight A pointer to a double to fill in with the height.
		void getPageSize(double *pWidth, double *pHeight);

		/// Get the page number of this page
		UInt32 getPageNumber() const;

		/// Returns the document this page belongs to
		NODocument getDocument() const;

		/// Copy over the resources from another page to this page
		/// @param pPage the Page to copy resources from
		void copyResourcesFrom(const NOPage &pPage);

		/// Normalize the content stream
		void normalizeContentStream();

		/// get the entry point to the object model for this page
		NOXFixedPage getFixedPage();

		/// create a Fixed page element, this is the entry point to the object model for this page
		NOXFixedPage createFixedPage();

		/// Renders a page to an image buffer of specified width and height
		/// The renderer will put the image centralized in the image buffer if the aspect ratio of the buffer does
		/// not correspond to the aspect ratio of the page.
		/// @pBuffer Image buffer large enough to hold 3 bytes per pixel for an image of the specified width and height
		/// @pWidth The width of the image to render to
		/// @pHeight The height of the image to render to
		/// @pAntiAliasing True or false indicating whether we should do (computationally intensive) antialiasing
		/// @pOutlineMode True or false controls rendering in outline mode
		UInt32 renderToImage(UInt8 *pBuffer, UInt32 pWidth, UInt32 pHeight, bool pAntiAliasing, bool pOutlineMode=false);
		
		UInt32 renderToImageDirect(UInt8 *pBuffer, UInt32 pWidth, UInt32 pHeight);

		/// Renders a part of a page to an image buffer of specified width and height
		/// The renderer will put the image centralized in the image buffer if the aspect ratio of the buffer does
		/// not correspond to the aspect ratio of the part of the page.
		/// @pBuffer Image buffer large enough to hold 3 bytes per pixel for an image of the specified width and height
		/// @pWidth The width of the image to render to
		/// @pHeight The height of the image to render to
		/// @pXPSRect The rectangle in XPS space to render
		/// @pAntiAliasing True or false indicating whether we should do (computationally intensive) antialiasing
		/// @pOutlineMode True or false controls rendering in outline mode
		UInt32 renderToImagePartial(UInt8 *pImageBuffer, UInt32 pWidth, UInt32 pHeight, doubleRect pXPSRect, bool pAntiAliasing, bool pOutlineMode);

		/// Renders a page to an tiff file of specified width and height
		/// The renderer will put the image centralized if the aspect ratio of the buffer does
		/// not correspond to the aspect ratio of the page.
		/// @pFile Output Tiff file
		/// @pWidth The width of the image to render to
		/// @pHeight The height of the image to render to
		/// @pResolutionInPixelsPerInch Resolution to be used in the file
		/// @pAntiAliasing True or false indicating whether we should do (computationally intensive) antialiasing
		UInt32 renderToTiff(const UTF8Char *pFile, UInt32 pWidth, UInt32 pHeight, UInt32 pResolutionInPixelsPerInch, UInt32 pAntiAliasing);

		UInt32 renderToGDI(void *pHDC/*, double pDPI*/);

		/// Exports a page to an SVG file
		/// @pFile Output SVG file
		UInt32 exportToSVG(const UTF8Char *pFile);

		/// Mark the current content of the page as a checkpoint for a future undo/redo
		void checkpoint();

		/// Undo to last checkpoint, returns success or failure
		bool undo();

		/// Redo to the next checkpoint, returns success or failure
		bool redo();
		
		/// Find the topmost object at pX horizontal and pY vertical considering the page
		/// is rendered in an image of pWidth by pHeight pixels.
		/// @pX The X-coordinate of the object to find
		/// @pY The Y-coordinate of the object to find
		/// @pWidth The width of the image to consider the coordinates in
		/// @pHeight The height of the image to consider the coordinates in
		UInt32 findObjectAt(UInt32 pX, UInt32 pY, UInt32 pWidth, UInt32 pHeight);
		
		/// Get object with Object ID @param pID as an NOBase object
		/// @pID The ID of the object to get
		NOBase getObject(UInt32 pID);
		
		/// Return all text objects in the same paragraph as @param pObjectID
		NOSelection getObjectsInParagraph(UInt32 pObjectID);
		
		/// Find text and select its pOccurrence'th occurrence
		/// @pText Text to find and select
		/// @pOccurrence Which occurrence to find and select
		bool findAndSelectText(const UTF8Char *pText, UInt32 pOccurrence);
		
		/// Find text and select all occureances
		/// @pText Text to find and select
		bool findAndSelectTextAll(const UTF8Char *pText);
	
		/// Return whether or not there is a current text selection on the page
		bool hasTextSelection();
		
		/// Reset the text selection of the page
		void clearTextSelection();
		
		/// get properties of a text object, like the bounding polygon (could be a rotate rectangle!)
		// and the point size
		void getTextProperties(UInt32 pTextObjectID,
					double &pX1, double &pY1,
					double &pX2, double &pY2,
					double &pX3, double &pY3,
					double &pX4, double &pY4,
					double &pSize);
		
		NOXStoryFragments getStoryFragments();		

		/// Select all text in the rectangle in XPS coordinates indicated by pTextRectangle
		/// This rectangle is interpreted as a text rectangle, so selecting a line can be done by dragging a narrow
		/// rectangle at the beginning of a line down to the beginning of the next line
		void selectTextMarquee(const doubleRect &pTextRectangle);
		
		/// Return the object ID for the selected text
		UInt32 getIDForTextSelection();
		
		/// Replace the text in the object with ID pTextObjectID so that 
		void replaceText(UInt32 pTextObjectID, const UTF8Char *pSearch, const UTF8Char *pReplace);
		
		/// Return an NOTextSelection object representing the current state of text selection
		NOTextSelection getTextSelection();

		/// Return the bounding box of an object in XPS-coordinate space
		doubleRect getBoundingBox(UInt32 pId);
		
		/// Move the object with ID pId by pDeltaX, pDeltaY points
		void moveObject(UInt32 pId, double pDeltaX, double pDeltaY);
		
		/// Remove the object with ID pId from the page
		void removeObject(UInt32 pId);
		
		/// Gets a copy of the selection.  The returned object allows to query
		/// the selection.
		NOSelection getCopyOfSelection();
		
		/// Set the selected state of object with ID pId to be pIsSelected
		void setSelection(UInt32 pId, bool pIsSelected);
		
		/// Invert the selected state of the object with ID pId
		void swapSelection(UInt32 pId);
		
		/// Clear the selection on this page
		void clearSelection();
		
		/// Select the objects within a rectangle considering the page is rendered in an
		/// image of pWidth by pHeight pixels
		/// @param pSwap When set to true it will not change the current selection but instead
		///		swap all the objects' selected state
		void selectMarquee( const UInt32Rect &pRect,
							NCommon::UInt32 pWidth,
							NCommon::UInt32 pHeight,
							bool pSwap);

		void selectMarquee(doubleRect pRect, bool pSwap, bool pIntersect);

		/// Select everything on the page		   
		void selectAll();
		
		/// Checks if a given object is transparent
		bool isTransparentObject(NCommon::UInt32 pId);
	
		/// Checks if a given object is rectangular
		bool isRectangularObject(NCommon::UInt32 pId);

		/// copies the content of a page as a layer on top of this current page
		/// returns an interface to the inserted page content 
		NOXCanvas copyPageOnTop(const NOPage &pPage);

		/// flattens all transparancy on the page
		bool flattenTransparency();
	
};

}
#endif // NIXPS_CPP_INTERFACE

#endif
