// NOSelection
//
// (c) NiXPS NV, January 2008

#ifndef _NOSELECTION_H_
#define _NOSELECTION_H_

#include "NOXPrefix.h"

#include "NOBase.h"

/*! \file NOSelection.h
    \brief Header file for NOSelection class
*/

#include "NTypes.h"

extern "C" {
	DLLIMPORTEXPORT NOSelectionHandler NOSelection_new(NOPageHandler *pPageHandler);
	DLLIMPORTEXPORT void NOSelection_delete(NOSelectionHandler *pHandler);
	DLLIMPORTEXPORT void NOSelection_getSelectedObject(NOSelectionHandler *pHandler, NCommon::UInt32 pIndex, NOXHandler *pObject);
	DLLIMPORTEXPORT NCommon::UInt32 NOSelection_getSelectedObjectID(NOSelectionHandler *pHandler, NCommon::UInt32 pIndex);
	DLLIMPORTEXPORT NCommon::UInt32 NOSelection_getNumberOfSelectedObjects(NOSelectionHandler *pHandler);
	DLLIMPORTEXPORT NCommon::UInt32 NOSelection_hasObject(						NOSelectionHandler *pHandler,
																	NCommon::UInt32 pIndex				);
	DLLIMPORTEXPORT void NOSelection_copy(NOSelectionHandler *pDestination, NOSelectionHandler *pHandler);
	DLLIMPORTEXPORT void NOSelection_removeObjectFromSelection(NOSelectionHandler *pHandler, NCommon::UInt32 pIndex);
};

#ifdef NIXPS_CPP_INTERFACE
namespace NiXPSObjects
{

using namespace NCommon;

class NOPartBased;
class NODocument;

/*!
 * \class NOSelection NOSelection.h "NOSelection.h"
 * Class to manage selected objects on a page.  Selected objects are the ones that are shown with their bounding
 * box in green in NiXPS Edit.  This is not the same as textselections where it is possible to select only a part
 * of a text object.
 */
class NOSelection {
	public:
	NOSelectionHandler mHandler;
	
	virtual ~NOSelection();

	NOSelection(NOSelectionHandler pHandler);
	
	NOSelection();

	/// Gets the C-safe NOSelectionHandler from this object
	// NOSelectionHandler &mHandler;
	
	/// Return the number of selected objects
	NCommon::UInt32 getNumberOfSelectedObjects();
	
	/// Get selected object's ID at index pIndex
	NCommon::UInt32 getSelectedObjectID(NCommon::UInt32 pIndex);
	
	/// Get selected object at index pIndex
	NiXPSObjects::NOBase getSelectedObject(NCommon::UInt32 pIndex);
	
	/// Convenience method for quickly checking whether the selection contains object pId
	bool hasObject(NCommon::UInt32 pId);
	
	void removeObjectFromSelection(NCommon::UInt32 pId);
	
	NOSelection &operator=(const NOSelection &pSelection);
	NOSelection(const NOSelection &pSelection);
};

}

#endif // NIXPS_CPP_INTERFACE
#endif	// _NOSELECTION_H_
