// NOString
//
// (c) NiXPS NV, May 2007

#ifndef _NOSTRING_H_
#define _NOSTRING_H_

/*! \file NOString.h
    \brief Header file for NOString class
*/

#include "NTypes.h"

extern "C" {
	DLLIMPORTEXPORT NOStringHandler NOString_new();
	DLLIMPORTEXPORT NOStringHandler NOString_newWithCString(NCommon::UTF8Char *pStr);
	DLLIMPORTEXPORT void NOString_delete(NOStringHandler *pString);
	DLLIMPORTEXPORT NCommon::UTF8Char* NOString_c_str(NOStringHandler *pString);
	DLLIMPORTEXPORT NCommon::UInt32 NOString_size(NOStringHandler *pString);
};

#ifdef NIXPS_CPP_INTERFACE
namespace NiXPSObjects
{

using namespace NCommon;

/*!
 * \class NOString NOString.h "NOString.h"
 * String class for NiXPS Library
 */
class NOString
{
	void initializeFrom(const NOString &pArgument);

	public:
		mutable NOStringHandler mHandler;

		// constructor: object is always linked to a specific package (and probably also a specific part)
		NOString();

		NOString(const UTF8String &pIn);
		NOString(NOStringHandler &pHandler);
	
		NOString(const NOString &pArgument);
		NOString &operator=(const NOString &pArgument);

		// destructor
		virtual ~NOString();
		
		//Cast to NOStringHandler operator
		operator NOStringHandler &() 
		{
			return mHandler;
		}

		// get handler
		NOStringHandler getHandler();

		/// Returns a UTF-8 encoded string
		const UTF8String getUTF8String() const;
		
		/// returns the size of the string
		UInt32 size();
};

}

#endif // NIXPS_CPP_INTERFACE

#endif
