// NOTextSelection
//
// (c) NiXPS NV, January 2008

#ifndef _NOTEXTSELECTION_H_
#define _NOTEXTSELECTION_H_

/*! \file NOSelection.h
    \brief Header file for NOSelection class
*/

#include "NTypes.h"
#include "NOString.h"

extern "C" {
	DLLIMPORTEXPORT NOTextSelectionHandler NOTextSelection_new();
	DLLIMPORTEXPORT void NOTextSelection_addSelection(	NOTextSelectionHandler *pHandler,
														NCommon::UInt32 pId,
														NCommon::doubleMatrixC pMatrix,
														NCommon::doubleRectC pRect,
														NOStringHandler *pString,
														NCommon::UInt32 pFirstChar,
														NCommon::UInt32 pLastChar);
	DLLIMPORTEXPORT void NOTextSelection_getSelection(	NOTextSelectionHandler *pHandler,
														NCommon::UInt32 pIndex,
														NCommon::UInt32 *pId,
														NCommon::doubleMatrixC *pMatrix,
														NCommon::doubleRectC *pRect);
	DLLIMPORTEXPORT NCommon::UInt32 NOTextSelection_getNumberOfSelections(NOTextSelectionHandler *pHandler);
	DLLIMPORTEXPORT void NOTextSelection_delete(NOTextSelectionHandler *pHandler);
	DLLIMPORTEXPORT void NOTextSelection_copy(NOTextSelectionHandler *pDestination, NOTextSelectionHandler pSource);
	DLLIMPORTEXPORT NOStringHandler NOTextSelection_getSelectedTextPart(NOTextSelectionHandler *pHandler, NCommon::UInt32 pIndex);
};

#ifdef NIXPS_CPP_INTERFACE
namespace NiXPSObjects
{

using namespace NCommon;

class NOPartBased;
class NODocument;

/*!
 * \class NOTextSelection NOTextSelection.h "NOTextSelection.h"
 * Class to manage selected text objects on a page.  Text selections are what is shown with a yellow
 * mark box in NiXPS Edit.  This is not the same as object selections with NOSelection.  Text selections
 * are specifically for text objects and can be about a part of the text.  This part is indicated with
 * a transformed rectangle.
 * You can manipulate the text selection of a page using calls on NOPage, like findAndSelectText and
 * getTextSelection.
 */
class NOTextSelection {
	NOTextSelectionHandler mHandler;
	
	private:

	public:
		NOTextSelection();
		NOTextSelection(const NOTextSelectionHandler &pIn);

		NOTextSelection &operator=(const NOTextSelection &pSelection);
		NOTextSelection(const NOTextSelection &pSelection);

		virtual ~NOTextSelection();
		
		/// Set the selection to a given id with a matrix and a rectangle
		/// @param pId The id of the text object to set selected
		/// @param pMatrix The matrix that is relevant to the text selection.  You get this
		///			matrix from getSelection
		/// @param pRect The rectangle that is relevant to the text selection.  You get this
		///			rectangle from getSelection
		/// void setSelection(UInt32 pId, doubleMatrix pMatrix, doubleRect pRect);
	
		/// The number of text selections currently active.
		UInt32 getNumberOfSelections();

		/// Get selection with index pIndex
		/// @param pIndex The index of the selection to get
		/// @param pId on return will contain the ID of the selected object
		/// @param pMatrix The matrix of the relevant part of the selection
		/// @param pRect The rectangle of the relevant part of the selection
		void getSelection(UInt32 pIndex, UInt32 &pId, doubleMatrix &pMatrix, doubleRect &pRect);

		/// Returns the part of the text that is selected as a UTF-8 encoded string
		/// @param pIndex The index of the selection to get
		NiXPSObjects::NOString getSelectedTextPart(UInt32 pIndex);
};

}

#endif // NIXPS_CPP_INTERFACE
#endif	// _NOTEXTSELECTION_H_
