// NOUtility.h
//
// NOUtility: an image
//			
//
// (c) NiXPS NV, February 2008

#ifndef _NOUTILITY_H_
#define _NOUTILITY_H_

/*! \file NOUtility.h
    \brief Header file for NOUtility functions
*/

#include "NTypes.h"
#include "NTImageFormat.h"

#include "NOPackageMap.h"
#include "NOProgressReporter.h"

#include "NOStringVector.h"

extern "C"
{
	DLLIMPORTEXPORT void NOUtility_getCSVFileColumnNames(NCommon::UTF8Char *pPathToCSVFile, NOStringVectorHandler *pColumnNames);
	DLLIMPORTEXPORT NCommon::UInt32 NOUtility_tiff_diff(NCommon::UTF8Char *pTiff);
	DLLIMPORTEXPORT void NOUtility_disable_encoded_pdf();
	DLLIMPORTEXPORT void NOUtility_enable_encoded_pdf();
	DLLIMPORTEXPORT void NOUtility_write_tiff(NCommon::UInt8 *pBuffer, NCommon::UTF8Char *pOut, NCommon::UInt32 pOutSizeH, NCommon::UInt32 pOutSizeV, double pRes);
	DLLIMPORTEXPORT void NOUtility_getColorFromXPSString(NCommon::UTF8Char *pFill, double *r, double *g, double *b, double *alpha);
	DLLIMPORTEXPORT NCommon::UInt32 NOUtility_runScript(	NCommon::UInt8 *pXMLScript,
															NOPackageMapHandler *pInputFiles,
															NOPackageMapHandler *pOutputFiles,
															NOProgressReporterHandler pProgressReporter,
															NOStringHandler *pOutput);
	// deprecated DLLIMPORTEXPORT NCommon::UInt32 NOUtility_convertToXPS(NCommon::UTF8Char *pSource, NCommon::UTF8Char *pDestination);
	DLLIMPORTEXPORT void NOUtility_getSuitableFontForFamilyAndStyle(NOStringHandler* pString, NCommon::UTF8Char *pFamilyName, NCommon::UTF8Char *pStyleName, NOProgressReporterHandler pProgressReporter);
	DLLIMPORTEXPORT NCommon::UInt32 NOUtility_haveCachedFontList();
	DLLIMPORTEXPORT NCommon::UInt32 NOUtility_convertToXPSViaPrinter(NCommon::UTF8Char *pSource, NCommon::UTF8Char *pDestination, NCommon::UTF8Char *pPrinterOutputFile, NCommon::UTF8Char *pXPSPrinterDriverName, NCommon::UInt32 pTimeOut);
}

#ifdef NIXPS_CPP_INTERFACE
namespace NiXPSObjects
{

/*!
 * \namespace NiXPSObjects::NOUtility
 * Namespace with utility functions.
 */
class NOUtility
{
	public:
	/**
	 * Return how many column names are in the csv file at a given
	 * path and which are their column names.
	 */
	static void getCSVFileColumnNames(const NCommon::UTF8Char *pPathToFile, NiXPSObjects::NOStringVector &pColumnNames);
	
	/**
	 * Get rgb and alpha components from an XPS FillString, like the ones obtained from 
	 * calling NOXPath::getFill or NOXGlyphs::getFill
	 */
	static void getColorFromXPSString(const NCommon::UTF8Char *pFillString, double &r, double &g, double &b, double &alpha);

	static bool runScript(	NCommon::UInt8 *pXMLScript,
						NOPackageMap &pInputFiles,
						NOPackageMap &pOutputFiles,
						NOProgressReporter pReporter, 
						NCommon::UTF8String &pOutput
						);

	/// Attempts to convert the source file to xps, and write this out to the pdestination location
	/// Please note that in order for this to work:
	/// 1 - You need the special printer driver installed on your system
	/// 2 - You need to have an application installed on your system that can print the source format
	static bool convertToXPSViaPrinter(NCommon::UTF8Char *pSource, NCommon::UTF8Char *pDestination, NCommon::UTF8Char *pXPSPrinterDriverName, NCommon::UTF8Char *pPrinterOutputFile, NCommon::UInt32 pTimeOut);

	/**
	 * For a certain family and style find a system installed font file that contains a font like this, possibly as a candidate for replacement
	 */
	static NCommon::UTF8String getSuitableFontForFamilyAndStyle(const UTF8Char *pFamilyName, const UTF8Char *pStyleName, NOProgressReporter pProgressReporter);

	/**
	 * The first call to getSuitableFontForFamilyAndStyle will iterate over all the fonts
	 * in the system directory and that will take a while.  This call is to check whether that
	 * will happen or not.
	 */
	static NCommon::UInt32 haveCachedFontList();
};

}

#endif // NIXPS_CPP_INTERFACE

#endif	// _NOUTILITY_H_
