// NOVector
//
// (c) NiXPS NV, May 2007

#ifndef _NOVECTOR_H_
#define _NOVECTOR_H_

/*! \file NOVector.h
    \brief Header file for NOVector class
*/

#include "NTypes.h"

extern "C" {
	DLLIMPORTEXPORT NCommon::UInt32 NOVector_size(NOVectorHandler *pVector);
	DLLIMPORTEXPORT NCommon::UInt16 NOVector_getPartIDAtIndex(NOVectorHandler *pVector, const NCommon::UInt32 pIndex);
	DLLIMPORTEXPORT NOVectorHandler NOVector_new(NOPackageHandler pPackage);
	DLLIMPORTEXPORT void NOVector_delete(NOVectorHandler *pVector);
	DLLIMPORTEXPORT void NOVector_addPart(NOVectorHandler *pVector, NOHandler pPart);
	DLLIMPORTEXPORT void NOVector_add(NOVectorHandler *pVector, NOVectorHandler pOtherVector);
	DLLIMPORTEXPORT void NOVector_addPartID(NOVectorHandler *pVector, NCommon::UInt16 pPartID);
	DLLIMPORTEXPORT void NOVector_copy(NOVectorHandler *pDestination, NOVectorHandler pSource);
};

#ifdef NIXPS_CPP_INTERFACE

namespace NiXPSObjects
{

using namespace NCommon;

class NOPartBased;
class NOPackage;

/*!
 * \class NOVector NOVector.h "NOVector.h"
 * Base class for NOImageVector and NOFontVector, vector classes containing images and fonts.
 */
class NOVector
{
	private:
		/// Assign one vector object to another
		NOVector &operator=(const NOVector &pArgument);

	protected:
		void addPart(NOHandler mPart);
		void addPartID(const UInt16 mPartID);

	public:
		NOVectorHandler mHandler;

		/// constructor: object is always linked to a specific package (and probably also a specific part)
		NOVector(NiXPSObjects::NOPackage *pPackage);

		
		// destructor
		virtual ~NOVector();
		
		// get handler
		NOVectorHandler getHandler() const;
		
		/// Returns the size of the vector
		UInt32 size() const;

		// get id
		const UInt16 getPartIDAtIndex(const UInt32 pIndex) const;
};

}
#endif // NIXPS_CPP_INTERFACE

#endif
