// NOXMLPartBased.h
//
// NOXMLPartBased:	base class for all NiXPSObjects based on an xml part
//			
//
// (c) NiXPS NV, April 2007

#ifndef _NOXMLPARTBASED_H_
#define _NOXMLPARTBASED_H_

/*! \file NOXMLPartBased.h
    \brief Header file for NOXMLPartBased class
*/

#include "NOXPrefix.h"
#include "NOPartBased.h"
#include "NTPartType.h"
#include "NOContentStreamNavigator.h"
#include "NOPartBased.h"

extern "C" {
	DLLIMPORTEXPORT void NOXMLPartBased_getContentStream(NOXMLPartBasedHandler *pHandler, NCommon::UInt8 *pBuffer, NCommon::UInt32 pSize);
	DLLIMPORTEXPORT NCommon::UInt32 NOXMLPartBased_getContentStreamLength(NOXMLPartBasedHandler *pHandler);
	DLLIMPORTEXPORT void NOXMLPartBased_replaceContentStream(NOXMLPartBasedHandler *pHandler, NCommon::UInt8 *pBuffer, NCommon::UInt32 pLen);
	DLLIMPORTEXPORT NOContentStreamNavigatorHandler NOXMLPartBased_getContentStreamNavigator(NOXMLPartBasedHandler *pHandler);
	// DLLIMPORTEXPORT NCommon::UInt32 NOXMLPartBased_getNOXHandlerForObject(NOXMLPartBasedHandler *pHandler, NOXHandler *pNOXHandler, NCommon::UInt32 pObjectID, NCommon::NXType pType);
};

#ifdef NIXPS_CPP_INTERFACE
namespace NiXPSObjects
{

using namespace NCommon;

/*!
 * \class NOXMLPartBased NOXMLPartBased.h "NOXMLPartBased.h"
 * Class representing an XML-based part of an XPS-package.
 * Base class for all NiXPSObjects based on an xml part.
 */
class NOXMLPartBased: public NOPartBased
{
	protected:
		// constructor
		NOXMLPartBased();

	public:
	
		// constructor
		NOXMLPartBased(const NOHandler pHandler);

		// destructor
		virtual ~NOXMLPartBased();

		/// Get the content stream as a buffer of UTF-8 characters
		/// @param pBuffer The buffer to be filled
		/// @param pSize The size of the buffer
		void getContentStream(UInt8 *pBuffer, UInt32 pSize);

		/// Get the length of the content stream
		UInt32 getContentStreamLength();

		/// Replace the contentstream with another stream buffer
		/// @param pBuffer The buffer to replace the content stream with
		/// @param pLen The length of the buffer
		void replaceContentStream(const UInt8 *pBuffer, UInt32 pLen);

		/// Return a content stream navigator object that helps navigating through the content stream
		NOContentStreamNavigator getContentStreamNavigator();
};

}
#endif // NIXPS_CPP_INTERFACE
#endif
