// NOXPathGeometry header file
//
// (c) NDR 2007-2008, NiXPS NV, All rights reserved

#ifndef _NOXPathGeometry_h_
#define _NOXPathGeometry_h_

/*! \file NOXPathGeometry.h
	\brief Header file for the NOXPathGeometry class
*/


#include "NOXPrefix.h"

extern "C" {
DLLIMPORTEXPORT NOXHandler NAXPathGeometry_getPathGeometryTransform(NOXHandler *pHandler, NCommon::UInt32 pIndex);
DLLIMPORTEXPORT NCommon::UInt32 NAXPathGeometry_getNumPathGeometryTransform(NOXHandler *pHandler);
DLLIMPORTEXPORT NOXHandler NAXPathGeometry_createPathGeometryTransform(NOXHandler *pHandler, NCommon::UInt32 pIndex);
DLLIMPORTEXPORT NCommon::UInt32 NAXPathGeometry_deletePathGeometryTransform(NOXHandler *pHandler, NCommon::UInt32 pIndex);
DLLIMPORTEXPORT NOXHandler NAXPathGeometry_getPathFigure(NOXHandler *pHandler, NCommon::UInt32 pIndex);
DLLIMPORTEXPORT NCommon::UInt32 NAXPathGeometry_getNumPathFigure(NOXHandler *pHandler);
DLLIMPORTEXPORT NOXHandler NAXPathGeometry_createPathFigure(NOXHandler *pHandler, NCommon::UInt32 pIndex);
DLLIMPORTEXPORT NCommon::UInt32 NAXPathGeometry_deletePathFigure(NOXHandler *pHandler, NCommon::UInt32 pIndex);
DLLIMPORTEXPORT void NAXPathGeometry_getFigures(NOXHandler *pHandler, NOXAbbrGeom *pOut);
DLLIMPORTEXPORT NCommon::UInt32 NAXPathGeometry_hasFigures(NOXHandler *pHandler);
DLLIMPORTEXPORT NCommon::UInt32 NAXPathGeometry_removeFigures(NOXHandler *pHandler);
DLLIMPORTEXPORT void NAXPathGeometry_setFigures(NOXHandler *pHandler, NOXAbbrGeom pObject);
DLLIMPORTEXPORT void NAXPathGeometry_getFillRule(NOXHandler *pHandler, NOXFillRule *pOut);
DLLIMPORTEXPORT NCommon::UInt32 NAXPathGeometry_hasFillRule(NOXHandler *pHandler);
DLLIMPORTEXPORT NCommon::UInt32 NAXPathGeometry_removeFillRule(NOXHandler *pHandler);
DLLIMPORTEXPORT void NAXPathGeometry_setFillRule(NOXHandler *pHandler, NOXFillRule pObject);
DLLIMPORTEXPORT NOXFillRule NAXPathGeometry_getDefaultFillRule();
DLLIMPORTEXPORT void NAXPathGeometry_getTransform(NOXHandler *pHandler, NOXRscRefMatrix *pOut);
DLLIMPORTEXPORT NCommon::UInt32 NAXPathGeometry_hasTransform(NOXHandler *pHandler);
DLLIMPORTEXPORT NCommon::UInt32 NAXPathGeometry_removeTransform(NOXHandler *pHandler);
DLLIMPORTEXPORT void NAXPathGeometry_setTransform(NOXHandler *pHandler, NOXRscRefMatrix pObject);
DLLIMPORTEXPORT void NAXPathGeometry_intialize(NOXHandler *pHandler);
DLLIMPORTEXPORT NOXHandler NAXPathGeometry_getChild(NOXHandler *pHandler, NCommon::UInt32 pIndex);
DLLIMPORTEXPORT NCommon::UInt32 NAXPathGeometry_getNumChilds(NOXHandler *pHandler);
}

namespace NiXPSObjects
{

using namespace NCommon;

/*!
 * \class NOXPathGeometry NOXPathGeometry.h "NOXPrefix.h"
 * Class NOXPathGeometry
 */
class NOXPathGeometry
{
	public:
		NOXHandler mHandler;
		NOXPathGeometry()
		{
			NIXPS_TRACE;

			mHandler.mDoc=0;
			mHandler.mType=0;
			mHandler.mCursor=0;
			mHandler.mError=0;
		}
		NOXPathGeometry(NOXHandler pHandler): mHandler(pHandler) {}

		/// Get PathGeometryTransform element at index pIndex
		/// @param pIndex index of the element you want to get
		/*public*/ NOXCP_Transform NOXPathGeometry::getPathGeometryTransform(NCommon::UInt32 pIndex)
		{
			return NAXPathGeometry_getPathGeometryTransform(&mHandler, pIndex);
		}

		/// Calculate the number of PathGeometryTransform elements that are present
		NCommon::UInt32 NOXPathGeometry::getNumPathGeometryTransform()
		{
			return NAXPathGeometry_getNumPathGeometryTransform(&mHandler);
		}

		/// Create an PathGeometryTransform element at index pIndex
		/*public*/ NOXCP_Transform NOXPathGeometry::createPathGeometryTransform(NCommon::UInt32 pIndex)
		{
			return NAXPathGeometry_createPathGeometryTransform(&mHandler, pIndex);
		}

		/// Delete an PathGeometryTransform element at index pIndex
		NCommon::UInt32 NOXPathGeometry::deletePathGeometryTransform(NCommon::UInt32 pIndex)
		{
			return NAXPathGeometry_deletePathGeometryTransform(&mHandler, pIndex);
		}

		/// Get PathFigure element at index pIndex
		/// @param pIndex index of the element you want to get
		/*public*/ NOXPathFigure NOXPathGeometry::getPathFigure(NCommon::UInt32 pIndex)
		{
			return NAXPathGeometry_getPathFigure(&mHandler, pIndex);
		}

		/// Calculate the number of PathFigure elements that are present
		NCommon::UInt32 NOXPathGeometry::getNumPathFigure()
		{
			return NAXPathGeometry_getNumPathFigure(&mHandler);
		}

		/// Create an PathFigure element at index pIndex
		/*public*/ NOXPathFigure NOXPathGeometry::createPathFigure(NCommon::UInt32 pIndex)
		{
			return NAXPathGeometry_createPathFigure(&mHandler, pIndex);
		}

		/// Delete an PathFigure element at index pIndex
		NCommon::UInt32 NOXPathGeometry::deletePathFigure(NCommon::UInt32 pIndex)
		{
			return NAXPathGeometry_deletePathFigure(&mHandler, pIndex);
		}

		/// Get the Figures attribute
		void NOXPathGeometry::getFigures(NOString &pString)
		{
			NOXStringBase lString=pString.getHandler();
			NAXPathGeometry_getFigures(&mHandler, &lString);
			return;
		}

		/// Check for the availability of the Figures attribute
		bool NOXPathGeometry::hasFigures()
		{
			return (NAXPathGeometry_hasFigures(&mHandler)!=0);
		}

		/// Removes the Figures attribute
		bool NOXPathGeometry::removeFigures()
		{
			return (NAXPathGeometry_removeFigures(&mHandler)!=0);
		}

		/// Sets the Figures attribute
		void NOXPathGeometry::setFigures(NOXAbbrGeom pObject)
		{
			NAXPathGeometry_setFigures(&mHandler, pObject);
		}

		/// Get the FillRule attribute
		NOXFillRule NOXPathGeometry::getFillRule()
		{
			NOXFillRule lHandler;
			NAXPathGeometry_getFillRule(&mHandler, &lHandler);
			return lHandler;
		}

		/// Check for the availability of the FillRule attribute
		bool NOXPathGeometry::hasFillRule()
		{
			return (NAXPathGeometry_hasFillRule(&mHandler)!=0);
		}

		/// Removes the FillRule attribute
		bool NOXPathGeometry::removeFillRule()
		{
			return (NAXPathGeometry_removeFillRule(&mHandler)!=0);
		}

		/// Sets the FillRule attribute
		void NOXPathGeometry::setFillRule(NOXFillRule pObject)
		{
			NAXPathGeometry_setFillRule(&mHandler, pObject);
		}

		/// Get the default FillRule
		NOXFillRule NOXPathGeometry::getDefaultFillRule()
		{
			return NAXPathGeometry_getDefaultFillRule();
		}

		/// Get the Transform attribute
		void NOXPathGeometry::getTransform(NOString &pString)
		{
			NOXStringBase lString=pString.getHandler();
			NAXPathGeometry_getTransform(&mHandler, &lString);
			return;
		}

		/// Check for the availability of the Transform attribute
		bool NOXPathGeometry::hasTransform()
		{
			return (NAXPathGeometry_hasTransform(&mHandler)!=0);
		}

		/// Removes the Transform attribute
		bool NOXPathGeometry::removeTransform()
		{
			return (NAXPathGeometry_removeTransform(&mHandler)!=0);
		}

		/// Sets the Transform attribute
		void NOXPathGeometry::setTransform(NOXRscRefMatrix pObject)
		{
			NAXPathGeometry_setTransform(&mHandler, pObject);
		}

		/// Initialize
		/*public*/ void intialize()
		{
			NAXPathGeometry_intialize(&mHandler);
		}

		/// Get child element at index pIndex
		/// @param pIndex index of the element you want to get
		/*public*/ NOXBase NOXPathGeometry::getChild(NCommon::UInt32 pIndex)
		{
			return NAXPathGeometry_getChild(&mHandler, pIndex);
		}

		/// Calculate the number of child elements that are present
		NCommon::UInt32 NOXPathGeometry::getNumChilds()
		{
			return NAXPathGeometry_getNumChilds(&mHandler);
		}

};

}

#endif
