// NOXPolyBezierSegment header file
//
// (c) NDR 2007-2008, NiXPS NV, All rights reserved

#ifndef _NOXPolyBezierSegment_h_
#define _NOXPolyBezierSegment_h_

/*! \file NOXPolyBezierSegment.h
	\brief Header file for the NOXPolyBezierSegment class
*/


#include "NOXPrefix.h"

extern "C" {
DLLIMPORTEXPORT void NAXPolyBezierSegment_getPoints(NOXHandler *pHandler, NOXPoints *pOut);
DLLIMPORTEXPORT NCommon::UInt32 NAXPolyBezierSegment_hasPoints(NOXHandler *pHandler);
DLLIMPORTEXPORT NCommon::UInt32 NAXPolyBezierSegment_removePoints(NOXHandler *pHandler);
DLLIMPORTEXPORT void NAXPolyBezierSegment_setPoints(NOXHandler *pHandler, NOXPoints pObject);
DLLIMPORTEXPORT void NAXPolyBezierSegment_getIsStroked(NOXHandler *pHandler, NOXBoolean *pOut);
DLLIMPORTEXPORT NCommon::UInt32 NAXPolyBezierSegment_hasIsStroked(NOXHandler *pHandler);
DLLIMPORTEXPORT NCommon::UInt32 NAXPolyBezierSegment_removeIsStroked(NOXHandler *pHandler);
DLLIMPORTEXPORT void NAXPolyBezierSegment_setIsStroked(NOXHandler *pHandler, NOXBoolean pObject);
DLLIMPORTEXPORT NOXBoolean NAXPolyBezierSegment_getDefaultIsStroked();
DLLIMPORTEXPORT void NAXPolyBezierSegment_intialize(NOXHandler *pHandler, NOXPoints pPoints);
}

namespace NiXPSObjects
{

using namespace NCommon;

/*!
 * \class NOXPolyBezierSegment NOXPolyBezierSegment.h "NOXPrefix.h"
 * Class NOXPolyBezierSegment
 */
class NOXPolyBezierSegment
{
	public:
		NOXHandler mHandler;
		NOXPolyBezierSegment()
		{
			NIXPS_TRACE;

			mHandler.mDoc=0;
			mHandler.mType=0;
			mHandler.mCursor=0;
			mHandler.mError=0;
		}
		NOXPolyBezierSegment(NOXHandler pHandler): mHandler(pHandler) {}

		/// Get the Points attribute
		void NOXPolyBezierSegment::getPoints(NOString &pString)
		{
			NOXStringBase lString=pString.getHandler();
			NAXPolyBezierSegment_getPoints(&mHandler, &lString);
			return;
		}

		/// Check for the availability of the Points attribute
		bool NOXPolyBezierSegment::hasPoints()
		{
			return (NAXPolyBezierSegment_hasPoints(&mHandler)!=0);
		}

		/// Removes the Points attribute
		bool NOXPolyBezierSegment::removePoints()
		{
			return (NAXPolyBezierSegment_removePoints(&mHandler)!=0);
		}

		/// Sets the Points attribute
		void NOXPolyBezierSegment::setPoints(NOXPoints pObject)
		{
			NAXPolyBezierSegment_setPoints(&mHandler, pObject);
		}

		/// Get the IsStroked attribute
		NOXBoolean NOXPolyBezierSegment::getIsStroked()
		{
			NOXBoolean lHandler;
			NAXPolyBezierSegment_getIsStroked(&mHandler, &lHandler);
			return lHandler;
		}

		/// Check for the availability of the IsStroked attribute
		bool NOXPolyBezierSegment::hasIsStroked()
		{
			return (NAXPolyBezierSegment_hasIsStroked(&mHandler)!=0);
		}

		/// Removes the IsStroked attribute
		bool NOXPolyBezierSegment::removeIsStroked()
		{
			return (NAXPolyBezierSegment_removeIsStroked(&mHandler)!=0);
		}

		/// Sets the IsStroked attribute
		void NOXPolyBezierSegment::setIsStroked(NOXBoolean pObject)
		{
			NAXPolyBezierSegment_setIsStroked(&mHandler, pObject);
		}

		/// Get the default IsStroked
		NOXBoolean NOXPolyBezierSegment::getDefaultIsStroked()
		{
			return NAXPolyBezierSegment_getDefaultIsStroked();
		}

		/// Initialize
		/*public*/ void intialize(NOXPoints pPoints)
		{
			NAXPolyBezierSegment_intialize(&mHandler, pPoints);
		}

};

}

#endif
