// NPlugIn
//
// plug-in interface
//
// (c) NiXPS NV, Februari 2008

#include "NTypes.h"

#ifndef _NPlugIn_H_
#define _NPlugIn_H_

typedef struct
{
	NCommon::UInt32 mSize;
	NCommon::UInt32 mHostMajor;
	NCommon::UInt32 mHostMinor;
	NCommon::UInt32 mMajor;
	NCommon::UInt32 mMinor;
	NCommon::UTF8Char mPath[255];
	NCommon::UInt32 mPathLen;	
} NPInitStruct;

typedef struct
{
	NCommon::UInt32 mSize;
	NCommon::UInt32 mID;
	NCommon::UTF8Char mKey[15];
	NCommon::UInt32 mKeyLen;
	NCommon::UTF8Char mMenuName[128];
	NCommon::UInt32 mMenuNameLen;	
	NCommon::UInt32 mHasRubber;
	NCommon::UInt8 mToolBarIcon[16*1024];	
	NCommon::UInt32 mToolBarIconSize;
	NCommon::UInt8 mRubberCursor[16*1024];	
	NCommon::UInt32 mRubberCursorSize;
} NPInfoStruct;

typedef struct
{
	NCommon::UInt32 mSize;
	NCommon::UInt32 mEvent;
	NOPackageHandler *mPackage;
	NODocumentHandler *mDocument;
	NOPageHandler *mPage;
} NPEventStruct;

// view interface
typedef void (*NPViewSetForegroundColor) (void *pHandle, NCommon::UInt32 pRed, NCommon::UInt32 pGreen, NCommon::UInt32 pBlue, NCommon::UInt32 pAlpha);
typedef void (*NPViewSetBackgroundColor) (void *pHandle, NCommon::UInt32 pRed, NCommon::UInt32 pGreen, NCommon::UInt32 pBlue, NCommon::UInt32 pAlpha);
typedef void (*NPViewDrawRectangle) (void *pHandle, NCommon::UInt32 pX, NCommon::UInt32 pY, NCommon::UInt32 pWidth, NCommon::UInt32 pHeight);
typedef void (*NPViewDrawText) (void *pHandle, NCommon::UInt32 pX, NCommon::UInt32 pY, NCommon::UTF8Char *pText);
typedef void (*NPViewTransferDeviceToAbsolute) (void *pHandle, NCommon::Int32 *pX, NCommon::Int32 *pY);
typedef void (*NPViewTransferAbsoluteToLogical) (void *pHandle, NCommon::Int32 *pX, NCommon::Int32 *pY, NOPageHandler *pPage, NCommon::UInt32 *pW, NCommon::UInt32 *pH);
typedef	NCommon::UInt32 (*NPViewTransferLogicalToDevice) (void *pHandle, NCommon::doubleRectC &pIn);
typedef void (*NPViewGetPage) (void *pHandle, NOPageHandler *pPage);
typedef void (*NPViewPostNotification) (void *pHandle, NCommon::UInt32 pEvent);

typedef struct
{
	NCommon::UInt32 mSize;
	void *mHandle;

	NPViewSetForegroundColor SetForegroundColor;
	NPViewSetBackgroundColor SetBackgroundColor;
	NPViewDrawRectangle DrawRectangle;
	NPViewDrawText DrawText;

	NPViewTransferDeviceToAbsolute TransferDeviceToAbsolute;
	NPViewTransferAbsoluteToLogical TransferAbsoluteToLogical;
	NPViewTransferLogicalToDevice TransferLogicalToDevice;

	NPViewGetPage GetPage;
	NPViewPostNotification PostNotification;
} NPViewStruct;

// rubber interface
typedef void (*NPRubberActivateFunc) ();
typedef void (*NPRubberDeactivateFunc) ();
typedef void (*NPRubberOnLeftUpFunc) (NPViewStruct *pView, NCommon::Int32 pX, NCommon::Int32 pY, NCommon::Int32 pShiftDown);
typedef void (*NPRubberOnLeftDownFunc) (NPViewStruct *pView, NCommon::Int32 pX, NCommon::Int32 pY, NCommon::Int32 pShiftDown);
typedef void (*NPRubberOnMouseMovedFunc) (NPViewStruct *pView, NCommon::Int32 pX, NCommon::Int32 pY, NCommon::Int32 pShiftDown);
typedef void (*NPRubberOnPaintFunc) (NPViewStruct *pView);
typedef void (*NPRubberOnPaintDragFunc) (NPViewStruct *pView);
typedef void (*NPRubberOnKeyDownFunc) (NPViewStruct *pView);

typedef struct
{
	NCommon::UInt32 mSize;
	NPRubberActivateFunc Activate;
	NPRubberDeactivateFunc Deactivate;
	NPRubberOnLeftUpFunc OnLeftUp;
	NPRubberOnLeftDownFunc OnLeftDown;
	NPRubberOnMouseMovedFunc OnMouseMoved;
	NPRubberOnPaintFunc OnPaint;
	NPRubberOnPaintDragFunc OnPaintDrag;
	NPRubberOnKeyDownFunc OnKeyDown;
} NPRubberInterfaceStruct;

// main interface
typedef void (*NPOpenFunc) ();
typedef void (*NPCloseFunc) ();
typedef void (*NPEventFunc) (NPEventStruct* pEvent);

typedef struct
{
	NCommon::UInt32 mSize;
	NPOpenFunc Open;
	NPOpenFunc Close;
	NPEventFunc Event;
	NPRubberInterfaceStruct RubberInterface;
} NPInterfaceStruct;

typedef void (*NPInitFunc) (NPInitStruct *pInit);
typedef void (*NPDestroyFunc) ();
typedef void (*NPGetInfoFunc) (NPInfoStruct *pInterface);
typedef void (*NPGetInterfaceFunc) (NPInterfaceStruct *pInterface);
typedef void (*NPGetRubberInterfaceFunc) (NCommon::UInt32 pNum, NPInterfaceStruct *pInterface);

#endif