// NTCorePropertyType.h
//
// (c) NiXPS NV, April 2007

#ifndef _NTCOREPROPERTYTYPE_H_
#define _NTCOREPROPERTYTYPE_H_

/*! \file NTCorePropertyType.h
    \brief Header file for core property types
*/
namespace NCommon
{

namespace CoreProperty
{

// Core Properties: table 3-1 in OPX 0.85 
// NOTE NDR (04/11/06): do not change these numbers, the api depends on it being the same

enum Type
{
	Category=0,			/**< Open Packaging Conventions The category. This value is typically used by UI applications to create navigation controls. */
	ContentStatus=1,	/**< Open Packaging Conventions The status of the content. Example values include "Draft", "Reviewed", and "Final".  */
	ContentType=2,		/**< Open Packaging Conventions The type of content represented, generally defined by a specific use and intended audience. Example values include "Whitepaper", "Security Bulletin", and "Exam". (This property is distinct from MIME content types as defined in RFC 2616.)  */
	Created=3,			/**< Dublin Core Date of creation of the resource.  */
	Creator=4,			/**< Dublin Core An entity primarily responsible for making the content of the resource.  */
	Description=5,		/**< Dublin Core An account of the content of the resource. Description may include but is not limited to: an abstract, table of contents, reference to a graphical representation of content or a free-text account of the content.  */
	Identifier=6,		/**< Dublin Core An unambiguous reference to the resource within a given context.  */
	Keywords=7,			/**< Open Packaging Conventions A delimited set of keywords to support searching and indexing. This is typically a list of terms that are not available elsewhere in the properties.  */
	Language=8,			/**< Dublin Core A language of the intellectual content of the resource.  */
	LastModifiedBy=9,	/**< Open Packaging Conventions The user who performed the last modification. The identification is environment-specific and can consist, for example, of a name, email address, or employee ID. It is recommended that this value be as concise as possible.  */
	LastPrinted=10,		/**< Open Packaging Conventions The date and time of the last printing.  */
	Modified=11,		/**< Dublin Core Date on which the resource was changed.  */
	Revision=12,		/**< Open Packaging Conventions The revision number. This value indicates the number of saves or revisions. The application is responsible for updating this value after each revision.  */
	Subject=13,			/**< Dublin Core The topic of the content of the resource.  */
	Title=14,			/**< Dublin Core The name given to the resource.  */
	Version=15,			/**< Open Packaging Conventions The version number. This value is set by the user or by the application.  */
	MaximumNumberOfCorePropertyTypes = 16
};

/*! \typedef NCommon::CoreProperty::Type
 *  A description of the enum type.
 */

/*! \namespace NCommon::CoreProperty
 *  The namespace for the core property related types and methods
 */

}

};

#endif

