// Common Types
//
// Some common types
//
// (c) NiXPS NV, June 2006

#ifndef _NTYPES_
#define _NTYPES_

/*! \file NTypes.h
    \brief Header file for common NiXPS Library types
*/

// enable the following if you want a trace build
#if 0
#if macosx
//#define	NIXPS_TRACE	std::cout << __FUNCTION__ << std::endl << std::flush
#define	NIXPS_TRACE	std::cout << __FILE__ << ": " << __FUNCTION__ << /*", line " << __LINE__ <<*/ std::endl << std::flush
#define	NIXPS_TRACE_LINE	std::cout << __FILE__ << ": " << __FUNCTION__ << ", line " << __LINE__ << std::endl << std::flush
#define	NIXPS_TRACE_MESSAGE(x) std::cout << __FUNCTION__ << "@" << __LINE__ << ": " << x << std::endl << std::flush
#else
#include <windows.h>
#include <atlstr.h>
#include <sstream>
#define	NIXPS_TRACE				{ CString OutMsg; OutMsg.Format(L"%s: %s, line %d",_T(__FILE__),_T(__FUNCTION__),__LINE__);OutputDebugString(OutMsg);}
#define	NIXPS_TRACE_LINE		{ CString OutMsg; OutMsg.Format(L"%s: %s, line %d",_T(__FILE__),_T(__FUNCTION__),__LINE__);OutputDebugString(OutMsg);}
#define	NIXPS_TRACE_MESSAGE(x)	{ CString OutMsg; OutMsg.Format(L"%s: %s, line %d: %s",_T( __FILE__),_T(__FUNCTION__),__LINE__,CString(x));OutputDebugString(OutMsg);}
#endif
#else
#define NIXPS_TRACE
#define	NIXPS_TRACE_LINE
#define	NIXPS_TRACE_MESSAGE(x)
#endif

#ifndef macosx
	#define win32	1
	#pragma warning( disable : 4251 )
#endif

#include <string>
#include <vector>
#include <math.h>
#include <iostream>

// **************************************************************************************
// NDR 30/8/2007
//
// Memory Leak Checker: uncomment this line, and make sure mmgr.cpp is compiled in to use
// #include "mmgr.h"
//
// **************************************************************************************

/*
#ifdef win32
	#ifdef _FLAG
		#if _FLAG
		#define  __declspec(dllexport)
		#else
		#define  __declspec(dllimport)
		#endif
	#else
		#define 
	#endif
#else
	#define 
	#define DLLIMPORTEXPORT
#endif
*/
#ifdef NIXPS_NO_DLL
	#define DLLIMPORTEXPORT
	#define NIXPS_CPP_INTERFACE
#else
	#ifdef win32
		#ifdef NIXPS_BUILD_DLL
			#define DLLIMPORTEXPORT __declspec(dllexport)
		#else
			#define DLLIMPORTEXPORT __declspec(dllimport)
		#endif
	#else
		#define DLLIMPORTEXPORT
	#endif
	#define NIXPS_CPP_INTERFACE
#endif

#ifdef win32
#define strncasecmp _strnicmp
#define strcasecmp _stricmp
#define snprintf _snprintf_s
#define fmax	__max
#define fmin	__min
#define NPATHSEPARATOR	'\\'
#else
#define NPATHSEPARATOR	'/'
#endif

// event codes
#define N_none						'----'
#define N_activePackageChanged		'pkCh'
//#define N_activePackageModified		'pkMo'
#define N_newPackageAdded			'pkAd'
#define N_packageRemoved			'pkRe'

// Changed:		it points to another page
// Modified:	it still is the same page, but it was modified
#define N_activePageChanged			'paCh'
#define N_activePageModified		'paMo'

#define N_zoomFactorChanged			'viZo'

#define N_activeDocumentChanged		'doCh'
#define N_activeSelectionChanged	'seCh'
#define N_activeSelectionMoved		'seMo'
#define N_activeSelectionModified	'seMd'
#define N_fontsChanged				'foCh'

namespace NCommon
{
	typedef unsigned char 		UInt8;	/**< 8 bits, unsigned */
	typedef unsigned int 		UInt32; /**< 32 bits, unsigned */
	typedef unsigned short		UInt16; /**< 16 bits, unsigned */
	typedef unsigned long long	UInt64; /**< 64 bits, unsigned */

	typedef signed char			Int8; /**< 8 bits, signed */
	typedef int					Int32; /**< 32 bits, signed */
	typedef short				Int16; /**< 16 bits, signed */
	typedef long long			Int64; /**< 64 bits, signed */

	typedef std::string		UTF8String; /**< UTF-8 encoded string */

	typedef std::string		PlatformCString;
	typedef char			UTF8Char;
	
	typedef std::vector<UInt32>		UTF32CharVector;
	
	typedef enum {
		Development		= 0,
		Beta			= 1,
		ReleaseCandidate= 2,
		Release			= 3
	};

	struct doubleColor
	{
		double red;
		double green;
		double blue;
		double alpha;
		double cyan;
		double magenta;
		double yellow;
		double black;
		bool isCMYK;
	};

	struct doublePoint
	{
		double x;
		double y;
		
		doublePoint(double x1, double y1) { x = x1; y = y1; }
	};

	struct doubleMatrixC 
	{
		double xx; 
		double yx;
		double xy; 
		double yy;
		double x0; 
		double y0;
	};

	struct doubleMatrix 
	{
		double xx; 
		double yx;
		double xy; 
		double yy;
		double x0; 
		double y0;

		public:
			// scale the matrix with sx & sy	
			void scale(double s)
			{
					xx*=s;
					xy*=s;
					yx*=s;
					yy*=s;
					x0*=s;
					y0*=s;
			}

			// transform a point (x,y)
			void transform(double *x, double *y)
			{
				double new_x, new_y;

				new_x = (xx * *x + xy * *y);
				new_y = (yx * *x + yy * *y);

				*x = new_x + x0;
				*y = new_y + y0;
			}
			
			doubleMatrixC getDoubleMatrixC() const
			{
				doubleMatrixC lMatrix;
				lMatrix.xx=xx; 
				lMatrix.yx=yx;
				lMatrix.xy=xy; 
				lMatrix.yy=yy;
				lMatrix.x0=x0; 
				lMatrix.y0=y0;
				return lMatrix;
			}

			doubleMatrix & operator=(const doubleMatrixC &pdoubleMatrixC)
			{
				xx=pdoubleMatrixC.xx;
				xy=pdoubleMatrixC.xy;
				yx=pdoubleMatrixC.yx;
				yy=pdoubleMatrixC.yy;
				x0=pdoubleMatrixC.x0;
				y0=pdoubleMatrixC.y0;
				return *this;
			}
	};

	struct doubleRectC
	{
		double mTop;
		double mLeft;
		double mBottom;
		double mRight;
	};
	
	struct UInt32Rect;

	struct doubleRect
	{
		double mTop;
		double mLeft;
		double mBottom;
		double mRight;
		
		public:

		doubleRect(double x1, double y1, double x2, double y2)
		{
			mLeft = x1;
			mTop = y1;
			mRight = x2;
			mBottom = y2;
		}

		doubleRect(doubleRectC pRectangle)
		{
			mLeft = pRectangle.mLeft;
			mTop = pRectangle.mTop;
			mRight = pRectangle.mRight;
			mBottom = pRectangle.mBottom;
		}

		doubleRect()
		{
			mLeft = mTop = mRight = mBottom = 0.;
		}

		doubleRectC getDoubleRectC() const
		{
			doubleRectC lRec;
			lRec.mTop=mTop;
			lRec.mBottom=mBottom;
			lRec.mLeft=mLeft;
			lRec.mRight=mRight;
			return lRec;
		}

		doubleRect(const UInt32Rect &pRectangle);
		
		void setEmpty()
		{
			mTop = HUGE; mBottom = -HUGE;
			mLeft = HUGE; mRight = -HUGE;
		}
		
		bool isEmpty()
		{
			return mBottom <= mTop || mRight <= mLeft;
		}
		
		bool pointInRect(const doublePoint &point)
		{
			return mLeft < point.x && point.y < mBottom && point.x < mRight && point.y > mTop;
		}
		
		double getWidth() const
		{
			return mRight - mLeft;
		}
		
		double getHeight() const
		{
			return mBottom - mTop;
		}
		
		doubleRect &intersect(const doubleRect &pRect)
		{
			mLeft = mLeft > pRect.mLeft ? mLeft : pRect.mLeft;
			mRight = mRight < pRect.mRight ? mRight : pRect.mRight;
			mTop = mTop > pRect.mTop ? mTop : pRect.mTop;
			mBottom = mBottom < pRect.mBottom ? mBottom : pRect.mBottom;
			return *this;
		}

		doubleRect &join(const doubleRect &pRect)
		{
			mLeft = mLeft < pRect.mLeft ? mLeft : pRect.mLeft;
			mRight = mRight > pRect.mRight ? mRight : pRect.mRight;
			mTop = mTop < pRect.mTop ? mTop : pRect.mTop;
			mBottom = mBottom > pRect.mBottom ? mBottom : pRect.mBottom;
			return *this;
		}
		
		bool surrounds(const doubleRect &pRect) const
		{
			return mLeft < pRect.mLeft && mRight > pRect.mRight &&
				mTop < pRect.mTop && mBottom > pRect.mBottom;
		}

		bool surroundsEq(const doubleRect &pRect) const
		{
			return mLeft <= pRect.mLeft && mRight >= pRect.mRight &&
				mTop <= pRect.mTop && mBottom >= pRect.mBottom;
		}
		
		doubleRect operator*=(double pFactor)
		{
			return (*this)*pFactor;
		}
		
		doubleRect operator*(double pFactor) const
		{
			doubleRect lReturnRect(*this);
			lReturnRect.mLeft		*= pFactor;
			lReturnRect.mRight		*= pFactor;
			lReturnRect.mTop		*= pFactor;
			lReturnRect.mBottom		*= pFactor;
			return lReturnRect;
		}
		
		doubleRect operator/(double pFactor) const
		{
			doubleRect lReturnRect(*this);
			lReturnRect.mLeft		/= pFactor;
			lReturnRect.mRight		/= pFactor;
			lReturnRect.mTop		/= pFactor;
			lReturnRect.mBottom		/= pFactor;
			return lReturnRect;
		}
		
		void normalize()
		{
			if (mLeft > mRight)
			{
				double tmp = mRight;
				mRight = mLeft;
				mLeft = tmp;
			}
			if (mTop > mBottom)
			{
				double tmp = mBottom;
				mBottom = mTop;
				mTop = tmp;
			}
		}
	};

	struct UInt32RectC
	{
		UInt32 mTop;
		UInt32 mLeft;
		UInt32 mBottom;
		UInt32 mRight;
	};
		
	struct UInt32Rect
	{
		UInt32 mTop;
		UInt32 mLeft;
		UInt32 mBottom;
		UInt32 mRight;
		
		void setToContain(doubleRect pDoubleRect)
		{
			mTop = (UInt32) floor(pDoubleRect.mTop);
			mLeft = (UInt32) floor(pDoubleRect.mLeft);
			mRight = (UInt32) ceil(pDoubleRect.mRight);
			mBottom = (UInt32) ceil(pDoubleRect.mBottom);
		}

		bool surroundsEq(const UInt32Rect &pRect) const
		{
			return mLeft <= pRect.mLeft && mRight >= pRect.mRight &&
				mTop <= pRect.mTop && mBottom >= pRect.mBottom;
		}

		void intersect(const UInt32Rect &pRect)
		{
			mLeft = mLeft > pRect.mLeft ? mLeft : pRect.mLeft;
			mRight = mRight < pRect.mRight ? mRight : pRect.mRight;
			mTop = mTop > pRect.mTop ? mTop : pRect.mTop;
			mBottom = mBottom < pRect.mBottom ? mBottom : pRect.mBottom;
		}
		
		bool operator==(const UInt32Rect &pRectangle)
		{
			return mLeft == pRectangle.mLeft &&
					mRight == pRectangle.mRight &&
					mTop == pRectangle.mTop &&
					mBottom == pRectangle.mBottom;
		}

		UInt32Rect & operator=(const UInt32RectC &pRectangleC)
		{
			mLeft=pRectangleC.mLeft;
			mRight=pRectangleC.mRight;
			mTop=pRectangleC.mTop;
			mBottom=pRectangleC.mBottom;
			return *this;
		}

		UInt32 getWidth() const
		{
			return mRight - mLeft;
		}
		
		UInt32 getHeight() const
		{
			return mBottom - mTop;
		}
		
		UInt32RectC getUInt32RectC() const
		{
			UInt32RectC lRect;
			lRect.mTop=mTop;
			lRect.mLeft=mLeft;
			lRect.mBottom=mBottom;
			lRect.mRight=mRight;
			
			return lRect;
		}
		
	};
		
	inline doubleRect::doubleRect(const UInt32Rect &pRectangle)
	{
		mLeft = pRectangle.mLeft;
		mTop = pRectangle.mTop;
		mRight = pRectangle.mRight;
		mBottom = pRectangle.mBottom;
	}

	extern bool decodeEntities(const UTF8Char *pStringWithEntities, const UInt32 pLen, UTF8String &pString);
	extern void encodeEntities(const UTF8Char *pString, const UInt32 pLen, UTF8String &pStringWithEntities);

//	extern bool isWhite(const UInt32 &pCurrent);
	#define isWhite(x) ( x==' ' || x=='\t' || x=='\n' || x=='\r' )
};


/*! \namespace NCommon
 * Namespace defining common types for the NiXPS Library
 */

namespace NiXPSAccess {
	class NAPackage;
};

namespace NiXML {
	class XMLDoc;
};

#define NIXPS_CALL_CONST(x,y) \
	x lRet= (x) y (const_cast<NOHandler *>(&mHandler)); \
	if (mHandler.mError) \
		throw(NOException(mHandler)); \
	return lRet; \

extern "C"
{
	struct NOHandler 
	{
		NiXPSAccess::NAPackage *mPackage;
		NCommon::UInt32 mID;
		NCommon::UInt32 mType;
		NCommon::UInt32 mError;
	};

	struct NOContentStreamNavigatorHandler 
	{
		NiXPSAccess::NAPackage *mPackage;
		NCommon::UInt32 mID;
		NCommon::UInt32 mCursor;
		NCommon::UInt32 mError;
	};
	
	struct NOXHandler 
	{
		NiXML::XMLDoc *mDoc;
		NCommon::UInt32 mType;
		NCommon::UInt32 mCursor;
		NCommon::UInt32 mError;
	};
	
	typedef void (*NOReporterUpdateFunction)(void *pHandle, NCommon::UInt32 pValue, NCommon::UInt32 pMaximum);

	struct NOProgressReporterHandler {
		NOReporterUpdateFunction mFunction;
		void *mHandler;
	};
	
	typedef NOHandler NOPackageHandler; 
	typedef NOHandler NOPageHandler;
	typedef NOHandler NODocumentHandler;
	typedef NOHandler NOFontHandler;
	typedef NOHandler NOImageHandler;
	typedef NOHandler NOThumbnailHandler;
	typedef NOHandler NOCorePropertiesHandler;
	typedef NOHandler NOPartWithResourcesHandler;
	typedef NOHandler NOXMLPartBasedHandler;
	
	struct NOVectorHandler 
	{
		NiXPSAccess::NAPackage *mPackage;
		void *mPtr;
		NCommon::UInt32 mError;
	};
	
	typedef NOVectorHandler NOFontVectorHandler;
	typedef NOVectorHandler NOImageVectorHandler;

	struct NOVoidHandler 
	{
		void *mPtr;
		NCommon::UInt32 mError;
	};

	typedef NOVoidHandler NOStringHandler;
	typedef NOVoidHandler NOTextSelectionHandler;
	typedef NOVoidHandler NOStringVectorHandler;
	typedef NOVoidHandler NOPackageMapHandler;
	typedef NOVoidHandler NOOptionsHandler;
	
	typedef NOVoidHandler NOSelectionHandler;

	typedef NOXHandler NOXBase;
	typedef NCommon::Int32 NOXIntBase;			/**< an integer */

	typedef NCommon::UInt32 NOXBooleanBase;		/**< a boolean */
	typedef double NOXDoubleBase;				/**< a double */
	typedef NOStringHandler NOXStringBase;		/**< a string */
	typedef NOStringHandler NOXUriBase;			/**< a URI */

	typedef double NOXZeroOne;					/**< a value */
	typedef double NOXGEZero;					/**< a value >= 0 */
	typedef double NOXDouble;					/**< a double */
	typedef double NOXGEOne;					/**< a value >= 1 */
	typedef NCommon::UInt32 NOXBoolean;			/**< a boolean value */
	typedef NOStringHandler NOXPoint;			/**< a point */
	typedef NOStringHandler NOXPointGE0;		/**<a point greater or equal to 0 */
	typedef NOStringHandler NOXPoints;			/**< a list of point in the XPS context */
	typedef NOStringHandler NOXEvenArrayPos;	/**< an array */
	typedef NOStringHandler NOXArray;			/**< an array */
	typedef NOStringHandler NOXViewBox;			/**< a view box ("x y width height") */
	typedef NOStringHandler NOXContentBox;		/**< a content box ("x y width height") */
	typedef NOStringHandler NOXBleedBox			/**< a bleed box ("x y width height") */;
	typedef NOStringHandler NOXMatrix;			/**< a matrix ("xx xy yx yy offx offy") */
	typedef NOStringHandler NOXColor;			/**< a color */
	typedef NOStringHandler NOXCaretStops;		/**< caret stops */
	typedef NOStringHandler NOXIndices;			/**< Indices */
	typedef NOStringHandler NOXUnicodeString;	/**< a unicode string color */
	typedef NOStringHandler NOXAbbrGeomF;		/**< abbriviated geometry information */
	typedef NOStringHandler NOXAbbrGeom;		/**< abbriviated geometry information */
	typedef NOStringHandler NOXUriImage;		/**< a uri to an image */
	typedef NOStringHandler NOXCtxBmpImage;		/**<  */
	typedef NOStringHandler NOXUriCtxBmp;		/**<  */
	typedef NOStringHandler NOXRscRef;			/**< a reference to a resource */
	typedef NOStringHandler NOXRscRefColor;		/**< a reference to a color resource */
	typedef NOStringHandler NOXRscRefMatrix;	/**< a reference to a matrix resource */
	typedef NOStringHandler NOXRscRefAbbrGeomF;	/**< a reference to an abbriviated geometry information resource */
	typedef NOStringHandler NOXName;			/**< a name */
	typedef NOStringHandler NOXNameUnique;		/**< a name */

	// enums: 'ints' for now
	typedef NCommon::Int32 NOXStyleSimulations;	/**<  */
	typedef NCommon::Int32 NOXSweepDirection;	/**<  */
	typedef NCommon::Int32 NOXTileMode;			/**<  */
	typedef NCommon::Int32 NOXViewUnits;		/**<  */
	typedef NCommon::Int32 NOXClrIntMode;		/**<  */
	typedef NCommon::Int32 NOXSpreadMethod;		/**<  */
	typedef NCommon::Int32 NOXMappingMode;		/**<  */
	typedef NCommon::Int32 NOXFillRule;			/**<  */
	typedef NCommon::Int32 NOXDashCap;			/**<  */
	typedef NCommon::Int32 NOXLineCap;			/**<  */
	typedef NCommon::Int32 NOXLineJoin;			/**<  */
	typedef NCommon::Int32 NOXEdgeMode;			/**<  */
	typedef NCommon::Int32 NOXTableSpan;		/**<  */
	typedef NCommon::Int32 NOXIntGEOne;			/**<  */
	typedef NCommon::Int32 NOXFragmentType;		/**<  */
};

#endif
