// Code
//
// example plug-in code
//
// (c) NiXPS NV, Februari 2008

#include <string>
#include <cstdio>
#include <sstream>

#include "NTypes.h"
#include "Code.h"

#include <Carbon/Carbon.h>

// static vars
static IBNibRef		sNibRef;
static WindowRef	window;
static ControlRef	control;
static std::stringstream	sBuffer;

// function defs
static OSStatus		HandleNew();
static OSStatus		WindowEventHandler( EventHandlerCallRef inCaller, EventRef inEvent, void* inRefcon );

// functions
void initUI(const char *pPath)
{
    OSStatus                    err;

	std::string lPath(pPath);
	lPath.erase(lPath.rfind('/'));
	lPath+="/PlugInResources/";

	CFURLRef lURL=CFURLCreateWithFileSystemPath (NULL, CFStringCreateWithCString(NULL,lPath.c_str(),kCFStringEncodingUTF8), kCFURLPOSIXPathStyle, true);
	CFBundleRef inBundle=CFBundleCreate (NULL, lURL);

	err=CreateNibReferenceWithCFBundle(inBundle, CFSTR("main"), &sNibRef);
	require_noerr( err, CantGetNibRef );

    HandleNew();
    
	logMessage(pPath);
	
CantSetMenuBar:
CantGetNibRef:
    return; // err;
}

//--------------------------------------------------------------------------------------------
DEFINE_ONE_SHOT_HANDLER_GETTER( WindowEventHandler )

void logMessage(const char *pMessage)
{
	std::stringstream lStr;
	lStr << time(NULL) << ": " << pMessage << std::endl << sBuffer.str();
	sBuffer.str("");
	sBuffer << lStr.str();
	SetControlData(control, kControlEntireControl, kControlEditTextTextTag, sBuffer.str().length(), sBuffer.str().c_str());
};

void openUI()
{
	ShowWindow(window);
}

void closeUI()
{
	HideWindow(window);
}

//--------------------------------------------------------------------------------------------
static OSStatus
HandleNew()
{
    OSStatus                    err;
    static const EventTypeSpec    kWindowEvents[] =
    {
        { kEventClassCommand, kEventCommandProcess},
		{ kEventClassWindow,  kEventWindowClose }
    };
    
    // Create a window. "MainWindow" is the name of the window object. This name is set in 
    // InterfaceBuilder when the nib is created.
    err = CreateWindowFromNib( sNibRef, CFSTR("MainWindow"), &window );
//    require_noerr( err, CantCreateWindow );

    // Install a command handler on the window. We don't use this handler yet, but nearly all
    // Carbon apps will need to handle commands, so this saves everyone a little typing.
    InstallWindowEventHandler( window, GetWindowEventHandlerUPP(),
                               GetEventTypeCount( kWindowEvents ), kWindowEvents,
                               window, NULL );
/*    
      eventType.eventClass = kEventClassWindow;          // Set event class
      eventType.eventKind  = kEventWindowClose;          // Set event kind
      handlerUPP = NewEventHandlerUPP(DoWindowClose);    // Point to handler
      InstallWindowEventHandler (theWindow, handlerUPP,  // Install handler
                                 1, &eventType,
                                 NULL, NULL);
*/
    // Position new windows in a staggered arrangement on the main screen
    RepositionWindow( window, NULL, kWindowCascadeOnMainScreen );
    
	// draw some text
    ControlID   cntlID;//      = { 100 };
	cntlID.id = 100;
	cntlID.signature = 'GGGG';
	err = GetControlByID(window,&cntlID,&control);

    // The window was created hidden, so show it
    // ShowWindow( window );
    	
    return err;
}

//--------------------------------------------------------------------------------------------
static OSStatus
WindowEventHandler( EventHandlerCallRef inCaller, EventRef inEvent, void* inRefcon )
{
    OSStatus    err = eventNotHandledErr;
    
    switch ( GetEventClass( inEvent ) )
    {
        case kEventClassCommand:
        {
            HICommandExtended cmd;
            verify_noerr( GetEventParameter( inEvent, kEventParamDirectObject, typeHICommand, NULL, sizeof( cmd ), NULL, &cmd ) );
            
            switch ( GetEventKind( inEvent ) )
            {
                case kEventCommandProcess:
                    switch ( cmd.commandID )
                    {
                        // Add your own command-handling cases here
                        case 'pimp':
							sBuffer.str("");
							logMessage("clear");
							break;
                        default:
                            break;
                    }
                    break;
            }
            break;
        }
            
		case kEventClassWindow:
		{
			closeUI();
			err = noErr;
		} break;
       default:
            break;
    }
    
    return err;
}
