// NOContentStreamNavigator.cpp
//
// NOContentStreamNavigator: navigate the (raw, xml) contentstream
//			
// (c) NiXPS NV, April 2006

/*! \file NOContentStreamNavigator.cpp
    \brief Implementation of the NOContentStreamNavigator class
*/

#include "NOContentStreamNavigator.h"
#include "NOXMLPartBased.h"
#include "NOString.h"

namespace NiXPSObjects
{

// doC# public NiXPS.NOContentStreamNavigatorHandler mHandler;

NOContentStreamNavigator::NOContentStreamNavigator(NOContentStreamNavigatorHandler &pHandler)
{
	mHandler = pHandler;
}

NOContentStreamNavigator::NOContentStreamNavigator(NOXMLPartBased &pPart)
{
	mHandler = NOContentStreamNavigator_new(&pPart.mHandler);
}

NOContentStreamNavigator::~NOContentStreamNavigator()
{
}

// get the node info
void NOContentStreamNavigator::getNodeInfo(UInt32 &pPos, UInt32 &pLen)
{
	NOContentStreamNavigator_getNodeInfo(&mHandler,&pPos,&pLen);
}

// get the number of child nodes
NCommon::UInt32 NOContentStreamNavigator::getNumberOfChilds() const
{
	return NOContentStreamNavigator_getNumberOfChilds(&mHandler);
}						

// navigate to child node at index 'pNum'
bool NOContentStreamNavigator::toChild(const UInt32 pNum)
{
	return NOContentStreamNavigator_toChild(&mHandler,pNum);
}

bool NOContentStreamNavigator::toChild(const UTF8Char *pString, const bool pCaseSensitive, const bool pIgnorePrefix)
{
	NOString lString(const_cast<UTF8Char *>(pString));
	return NOContentStreamNavigator_toChildWithName(&mHandler,&lString.mHandler,pCaseSensitive,pIgnorePrefix);
}

// navigate to sibling
bool NOContentStreamNavigator::toSibling()
{
	return NOContentStreamNavigator_toSibling(&mHandler);
}

// navigate to parent
bool NOContentStreamNavigator::toParent()
{
	return NOContentStreamNavigator_toParent(&mHandler);
}

// get the name of the current node
const NCommon::UTF8String NOContentStreamNavigator::getName() const
{
	NOString lString;
	NOContentStreamNavigator_getName(&mHandler, &lString.mHandler);
	return lString.getUTF8String();
}						

// tests for the presence of the given attribute
bool NOContentStreamNavigator::hasAttributeValue(const UTF8Char *pKey) const
{
	NOString lString(const_cast<UTF8Char *>(pKey));
	return NOContentStreamNavigator_hasAttributeValue(&mHandler,&lString.mHandler);
}

// get the value of an attribute of the current node
const NCommon::UTF8String NOContentStreamNavigator::getAttributeValue(const UTF8Char *pKey) const
{
	NOString lString;
	NOString lKeyString(const_cast<UTF8Char *>(pKey));
	NOContentStreamNavigator_getAttributeValue(&mHandler,&lKeyString.mHandler,&lString.mHandler);
	return lString.getUTF8String();
}						

bool NOContentStreamNavigator::setAttributeValue(const UTF8Char *pKey, const UTF8Char *pValue)
{
	NOString lKeyString(const_cast<UTF8Char *>(pKey));
	NOString lValueString(const_cast<UTF8Char *>(pValue));
	return NOContentStreamNavigator_setAttributeValue(&mHandler,&lKeyString.mHandler,&lValueString.mHandler);
}

bool NOContentStreamNavigator::addAttribute(const UTF8Char *pKey, const UTF8Char *pValue)
{
	NOString lKeyString(const_cast<UTF8Char *>(pKey));
	NOString lValueString(const_cast<UTF8Char *>(pValue));
	return NOContentStreamNavigator_addAttribute(&mHandler,&lKeyString.mHandler,&lValueString.mHandler);
}

bool NOContentStreamNavigator::toAttributeValue(const UTF8Char *pKey)
{
	NOString lKeyString(const_cast<UTF8Char *>(pKey));
	return NOContentStreamNavigator_toAttributeValue(&mHandler,&lKeyString.mHandler);
}

// check to see if the current element has character data
bool NOContentStreamNavigator::hasCharacterData() const
{
	return NOContentStreamNavigator_hasCharacterData(&mHandler);
}						

// read the character data for the current node
const NCommon::UTF8String NOContentStreamNavigator::getCharacterData() const
{
	NOString lString;
	NOContentStreamNavigator_getCharacterData(&mHandler, &lString.mHandler);
	return lString.getUTF8String();
}

bool NOContentStreamNavigator::insertChild(const NCommon::UTF8Char *pName, NCommon::UInt32 pPosition, bool pToChild)
{
	NOString lString(const_cast<UTF8Char *>(pName));
	return NOContentStreamNavigator_insertChild(&mHandler, &lString.mHandler, pPosition, pToChild);
}

bool NOContentStreamNavigator::insertAsChild(const UTF8Char *pXML)
{
	NOString lString(const_cast<UTF8Char *>(pXML));
	return NOContentStreamNavigator_insertAsChild(&mHandler,&lString.mHandler);
}

bool NOContentStreamNavigator::insertAsChild(NOContentStreamNavigator &pNav)
{
	return NOContentStreamNavigator_insertAsChildWithNavigator(&mHandler,&pNav.mHandler);
}

bool NOContentStreamNavigator::toID(NCommon::UInt32 pID)
{
	return NOContentStreamNavigator_toID(&mHandler,pID);
}

}
