// NODocument.h
//
// NODocument: NODocument class
//			
//
// (c) NiXPS NV, October 2006

/*! \file NODocument.cpp
    \brief Implementation of the NODocument class
*/

#include "NTypes.h"
#include "NODocument.h"
#include "NOPage.h"
#include "NOException.h"

namespace NiXPSObjects
{

NODocument::NODocument(): NOPartWithResources() 
{
}

NODocument::NODocument(NODocumentHandler pHandler): NOPartWithResources(pHandler) 
{
};

NODocument::~NODocument() 
{
};

// returns the number of pages
NCommon::UInt32 NODocument::getNumberOfPages()
{
	UInt32 lRet=NODocument_getNumberOfPages(&mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return lRet;
};

// returns the given page (location pNum)
NOPage NODocument::getPage(const UInt32 pNum)
{
	NOPageHandler lHandler=NODocument_getPage(&mHandler, pNum);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return NOPage(lHandler);
};
		
// returns the page number this page has in the doc
NCommon::UInt32 NODocument::getPageNumberOf(const NOPage &pPage)
{
	UInt32 lRet=NODocument_getPageNumberOf(&mHandler, (NOPageHandler) pPage.mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));
		
	return lRet;
};
		
// insert a page into a given location
NOPage NODocument::insertPage(const NOPage &pPage, UInt32 pPos)
{
	NOPageHandler lHandler=NODocument_insertPage(&mHandler, (NOPageHandler) pPage.mHandler, pPos);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return NOPage(lHandler);
};

// creates a new page, adds this to the back of the pagelist
NOPage NODocument::createPage()
{
	NOPageHandler lHandler = NODocument_createPage(&mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return NOPage(lHandler);
}

/// get the entry point to the object model for the document structure
/*public*/ NOXDocumentStructure NODocument::getDocumentStructure()
{
	NOXDocumentStructure lHandler=NODocument_getDocumentStructure(&mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return lHandler; 
}

/// returns wheter the document has structure information
bool NODocument::hasDocumentStructure()
{
	UInt32 lRet=NODocument_hasDocumentStructure(&mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return lRet;
}

// search and replace text in a document
bool NODocument::searchAndReplaceText(const UTF8Char *pSearch, const UTF8Char *pReplace, NOFontVector &pMissingFonts, const bool pDryRun)
{
	UInt32 lRet=NODocument_searchAndReplaceText(&mHandler, const_cast<UTF8Char *>(pSearch), const_cast<UTF8Char *>(pReplace), pMissingFonts.mHandler, pDryRun);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return lRet;
}

UInt32 NODocument::exportToPDF(const UTF8Char *pPDFFileName, NOProgressReporter &pProgressReporter)
{
	UInt32 lRet = NODocument_exportToPDF(&mHandler,const_cast<UTF8Char *>(pPDFFileName),pProgressReporter.mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

UInt32 NODocument::exportToPDF(UInt8 *pBuffer, const UInt32 pBufSize, UInt32 &pPDFSize, NOProgressReporter &pProgressReporter)
{
	UInt32 lRet = NODocument_exportToPDFInMemory(&mHandler, pBuffer, pBufSize, &pPDFSize, pProgressReporter.mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

NOPackage NODocument::getPackage()
{
	NOPackageHandler lHandler = NODocument_getPackage(&mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return NOPackage(lHandler);
}

UInt32 NODocument::getDocumentNumber()
{
	UInt32 lRet = NODocument_getDocumentNumber(&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

NCommon::UInt32 NODocument::getFonts(NOFontVector &pFonts)
{
	UInt32 lRet = NODocument_getFonts(&mHandler, pFonts.mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return lRet;
}

NCommon::UInt32 NODocument::getImages(NOImageVector &pImages)
{
	UInt32 lRet = NODocument_getImages(&mHandler, pImages.mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return lRet;
}

UInt32 NODocument::resolveLink(const UTF8Char *pSearch, UInt16 *pPageID, UInt32 *pObjectID)
{
	UInt32 lRet = NODocument_resolveLink(&mHandler, const_cast<UTF8Char *>(pSearch), (UInt16 *)pPageID, (UInt32 *)pObjectID);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	
	return lRet;
}

};
