// NOFont.h
//
// NOFont: a font
//			
//
// (c) NiXPS NV, June 2007

/*! \file NOFont.cpp
    \brief Implementation of the NOFont class
*/

#include "NOException.h"
#include "NOString.h"
#include "NOPackage.h"

#include "NOFont.h"

namespace NiXPSObjects
{

// constructor: object is always linked to a specific package (and probably also a specific part)
NOFont::NOFont(NOFontHandler pHandler): NOPartBased(pHandler)
{
}

NOFont::NOFont(NOPackage *pPackage, UInt16 pID)
{
	mHandler = NOFont_new(&(pPackage->mHandler),pID);
}

// destructor
NOFont::~NOFont()
{
}

// returns the filename of the font
const NCommon::UTF8String NOFont::getFileName()
{
	NOString lString;
	NOFont_getFileName(&mHandler,&lString.mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return lString.getUTF8String();
}

// returns the family name of the font
const NCommon::UTF8String NOFont::getFontFamilyName()
{
	NOString lString;
	NOFont_getFontFamilyName(&mHandler,&lString.mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return lString.getUTF8String();
}

const NCommon::UTF8String NOFont::getFontStyleName()
{
	NOString lString;
	NOFont_getFontStyleName(&mHandler,&lString.mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return lString.getUTF8String();
}

bool NOFont::hasGlyphsFor(const UTF8Char *pTextString)
{
	NCommon::UInt32 lRet = NOFont_hasGlyphsFor(&mHandler,const_cast<UTF8Char *>(pTextString));

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return lRet;
}

// is a font restricted
NCommon::UInt32 NOFont::isRestricted()
{
	UInt32 lRet=NOFont_isRestricted(&mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return lRet;
}

// saves font to a file	
void NOFont::writeToFile(const UTF8Char *destinationPath)
{
	NOFont_writeToFile(&mHandler,const_cast<UTF8Char *>(destinationPath));

	if (mHandler.mError!=0)
		throw(NOException(mHandler));
}

// replaces a font with the given file
bool NOFont::replaceWithFile(const UTF8Char *destinationPath)
{
	UInt32 lRet=NOFont_replaceWithFile(&mHandler,const_cast<UTF8Char *>(destinationPath));

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return lRet;
}

}