// NOImageDLL.cpp
//
// NOImageDLL
//			
//
// (c) NiXPS NV, June 2007

/*! \file NOImage.cpp
    \brief Implementation of the NOImage class
*/

#define NIXPS_CPP_INTERFACE

#include "NOException.h"
#include "NOString.h"
#include "NOPackage.h"

#include "NOImage.h"

namespace NiXPSObjects
{

NOImage::NOImage(NOImageHandler pHandler): NOPartBased(pHandler)
{
}

NOImage::NOImage(NOPackage *pPackage, NCommon::UInt16 pID)
{
	mHandler = NOImage_new2(&(pPackage->mHandler), pID);
}

// destructor
NOImage::~NOImage()
{
}

// returns the filename of the image
const NCommon::UTF8String NOImage::getFileName()
{
	NOString lString;
	NOImage_getFileName(&mHandler,&lString.mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return lString.getUTF8String();
}

// gets the image type
NCommon::Image::Format NOImage::getFormat()
{
	Image::Format lRet=(Image::Format)NOImage_getFormat(&mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return lRet;
}

void NOImage::writeImageToFile(const UTF8Char *destinationPath)
{
	NOImage_writeImageToFile(&mHandler, const_cast<UTF8Char *>(destinationPath));

	if (mHandler.mError!=0)
		throw(NOException(mHandler));
}

// Fill pBuffer with the image data, and 
bool NOImage::getImageData(NCommon::UInt8 *pBuffer, NCommon::UInt32 pSize)
{
	UInt32 lRet=NOImage_getImageData(&mHandler, pBuffer, pSize);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return lRet;
}

// Return the size of the image in bytes (UInt8's)
NCommon::UInt32 NOImage::getImageSize() const
{
	NCommon::UInt32 lRet=NOImage_getImageSize((NOHandler *)&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

/// Returns the width of the image in pixels
NCommon::UInt32 NOImage::getWidthInPixels() const
{
	NCommon::UInt32 lRet=NOImage_getWidthInPixels((NOHandler *)&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

/// Returns the height of the image in pixels
NCommon::UInt32 NOImage::getHeightInPixels() const
{
	NCommon::UInt32 lRet=NOImage_getHeightInPixels((NOHandler *)&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

/// Returns resolution of the image in the X direction
double NOImage::getXResolution() const
{
	double lRet=NOImage_getXResolution((NOHandler *)&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

/// Returns resolution of the image in the Y direction
double NOImage::getYResolution() const
{
	double lRet=NOImage_getYResolution((NOHandler *)&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

/// Returns the colortype of the image
Image::ColorType NOImage::getColorType() const
{
	Image::ColorType lRet=(Image::ColorType)NOImage_getColorType((NOHandler *)&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

/// Returns the bitdepth of the image
Image::BitDepth NOImage::getBitDepth() const
{
	Image::BitDepth lRet=(Image::BitDepth)NOImage_getBitDepth((NOHandler *)&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}
}