// NOPage.cpp
//
// NOPage: NOPage class
//			
// (c) NiXPS NV, November 2006

/*! \file NOPage.cpp
    \brief Implementation of the NOPage class
*/

#include "NOXPrefix.h"
#include "NODocument.h"
#include "NOPage.h"
#include "NOException.h"

namespace NiXPSObjects
{

NOPage::NOPage(): NOPartWithResources() 
{
}

NOPage::NOPage(const NOPageHandler pHandler): NOPartWithResources(pHandler) 
{
}

NOPage::NOPage(NOPackage *pPackage, NCommon::UInt16 pPageID)
{
	mHandler = NOPage_new(&(pPackage->mHandler), pPageID);
}

// destructor
NOPage::~NOPage()
{
}

NCommon::UInt32 NOPage::getFonts(NOFontVector &pFonts)
{
	UInt32 lRet=NOPage_getFonts(&mHandler, pFonts.mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return lRet;
}

NOFont NOPage::getFont(const UTF8Char *pURI)
{
	NOFontHandler lHandler = NOPage_getFont(&mHandler, const_cast<UTF8Char *>(pURI));

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return NOFont(lHandler);
}

NOImage NOPage::getImage(const UTF8Char *pURI)
{
	NOImageHandler lHandler = NOPage_getImage(&mHandler, const_cast<UTF8Char *>(pURI));

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return NOImage(lHandler);
}

// returns the count of the number of images and a vector with all the images
NCommon::UInt32 NOPage::getImages(NOImageVector &pImages)
{
	UInt32 lRet=NOPage_getImages(&mHandler, pImages.mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return lRet;
}
		
// returns the document this page belongs to
NODocument NOPage::getDocument() const
{
	NODocumentHandler lHandler = NOPage_getDocument((NOPageHandler *) &mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return NODocument(lHandler);
}

// add a font to this page, returns a handler to the new font
NOFont NOPage::addFont(const NOFont &pFont)
{
	NOFontHandler lHandler=NOPage_addFont((NOFontHandler *) &mHandler, pFont.mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return NOFont(lHandler);
}

// add a font to this page, returns a handler to the new font
NOFont NOPage::addFont(const UTF8Char *pFontFile)
{
	NOFontHandler lHandler=NOPage_addFontFile((NOFontHandler *) &mHandler, const_cast<UTF8Char *>(pFontFile));

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return NOFont(lHandler);
}

// add an image font to this page, returns a handler to the new image
NOImage NOPage::addImage(const NOImage &pImage)
{
	NOImageHandler lHandler=NOPage_addImage((NOImageHandler *) &mHandler, pImage.mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return NOImage(lHandler);
}

// add an image to this page, returns a handler to the new image
NOImage NOPage::addImage(const UTF8Char *pImageFile)
{
	NOFontHandler lHandler=NOPage_addImageFile((NOFontHandler *) &mHandler, const_cast<UTF8Char *>(pImageFile));

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return NOImage(lHandler);
}

NOPackage NOPage::getPackage()
{
	NOPackageHandler lHandler = mHandler;
	return NOPackage(mHandler);
}

// copy over the resources from a given other page
void NOPage::copyResourcesFrom(const NOPage &pPage)
{
	NOPage_copyResourcesFrom(&mHandler, (NOPageHandler *) &pPage.mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));
}

// normalize the content stream
void NOPage::normalizeContentStream()
{
	NOPage_normalizeContentStream(&mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));
}
		
// get the page size
void NOPage::getPageSize(double *pWidth, double *pHeight)
{
	NOPage_getPageSize(&mHandler, (double *) pWidth, (double *) pHeight);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));
}

// get the page number
NCommon::UInt32 NOPage::getPageNumber() const
{
	UInt32 lRet=NOPage_getPageNumber((NOPageHandler *) &mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return lRet;
}

// returns a handler to the content (entry point into the XSD world)
/*public*/ NOXFixedPage NOPage::getFixedPage()
{
	NOXFixedPage lHandler=NOPage_getFixedPage(&mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return lHandler;
}

// creates content (entry point into the XSD world)
/*public*/ NOXFixedPage NOPage::createFixedPage()
{
	NOXFixedPage lHandler=NOPage_createFixedPage(&mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return lHandler;
}

UInt32 NOPage::renderToImage(UInt8 *pImageBuffer, UInt32 pWidth, UInt32 pHeight, bool pAntiAliasing, bool pOutlineMode)
{
	UInt32 lRet = NOPage_renderToImage(&mHandler,pImageBuffer,pWidth,pHeight,pAntiAliasing,pOutlineMode);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

UInt32 NOPage::renderToImageDirect(UInt8 *pImageBuffer, UInt32 pWidth, UInt32 pHeight)
{
	UInt32 lRet = NOPage_renderToImageDirect(&mHandler,pImageBuffer,pWidth,pHeight);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

#if win32
UInt32 NOPage::renderToGDI(void * pHDC/*, double pDPI*/)
{
	UInt32 lRet = NOPage_renderToGDI(&mHandler,pHDC/*,pDPI*/);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}
#endif

UInt32 NOPage::renderToImagePartial(UInt8 *pImageBuffer, UInt32 pWidth, UInt32 pHeight, doubleRect pXPSRect, bool pAntiAliasing, bool pOutlineMode)
{
	doubleRectC lXPSRectC;
	lXPSRectC.mTop = pXPSRect.mTop;
	lXPSRectC.mLeft = pXPSRect.mLeft;
	lXPSRectC.mRight = pXPSRect.mRight;
	lXPSRectC.mBottom = pXPSRect.mBottom;
	UInt32 lRet = NOPage_renderToImagePartial(&mHandler,pImageBuffer,pWidth,pHeight,lXPSRectC,pAntiAliasing,pOutlineMode);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

UInt32 NOPage::renderToTiff(const UTF8Char *pFile, UInt32 pWidth, UInt32 pHeight, UInt32 pResolutionInPixelsPerInch, UInt32 pAntiAliasing)
{
	UInt32 lRet =  NOPage_renderToTiff(&mHandler, const_cast<UTF8Char *>(pFile), pWidth, pHeight, pResolutionInPixelsPerInch, pAntiAliasing);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

UInt32 NOPage::exportToSVG(const UTF8Char *pFile)
{
	UInt32 lRet =  NOPage_exportToSVG(&mHandler, const_cast<UTF8Char *>(pFile));
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

void NOPage::checkpoint()
{
	/*UInt32 lRet =  */NOPage_checkpoint(&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	//return lRet;
}

bool NOPage::undo()
{
	UInt32 lRet = NOPage_undo(&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

bool NOPage::redo()
{
	UInt32 lRet = NOPage_redo(&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

UInt32 NOPage::findObjectAt(UInt32 pX, UInt32 pY, UInt32 pWidth, UInt32 pHeight)
{
	UInt32 lRet = NOPage_findObjectAt(&mHandler,pX,pY,pWidth,pHeight);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

NOBase NOPage::getObject(UInt32 pID)
{
	NOXHandler lHandler;
	NOPage_getObject(&mHandler,pID,&lHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return NOBase(lHandler);
}

NOXStoryFragments NOPage::getStoryFragments()
{
	NOXStoryFragments lHandler=NOPage_getStoryFragments(&mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return lHandler;
}

bool NOPage::findAndSelectText(const UTF8Char *lText, UInt32 pOccurrence)
{
	bool lRet = NOPage_findAndSelectText(&mHandler,const_cast<UTF8Char *>(lText),pOccurrence);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}

bool NOPage::findAndSelectTextAll(const UTF8Char *lText)
{
	bool lRet = NOPage_findAndSelectTextAll(&mHandler,const_cast<UTF8Char *>(lText));
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}
	
bool NOPage::hasTextSelection()
{
	bool lRet = NOPage_hasTextSelection(&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}
		
void NOPage::clearTextSelection()
{
	NOPage_clearTextSelection(&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
}

UInt32 NOPage::getIDForTextSelection()
{
	UInt32 lReturnValue = NOPage_getIDForTextSelection(&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lReturnValue;
}

void NOPage::replaceText(UInt32 pTextObjectID, const UTF8Char *pSearch, const UTF8Char *pReplace)
{
	NOPage_replaceText(&mHandler,pTextObjectID, const_cast<UTF8Char *>(pSearch), const_cast<UTF8Char *>(pReplace));
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
}

NOTextSelection NOPage::getTextSelection()
{
	NOTextSelectionHandler lTextSelection = NOPage_getTextSelection(&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return NOTextSelection(lTextSelection);
}

void NOPage::selectTextMarquee(const doubleRect &pTextRectangle)
{
	NOPage_selectTextMarquee(&mHandler,pTextRectangle.getDoubleRectC());
}

NCommon::doubleRect NOPage::getBoundingBox(UInt32 pId)
{
	NCommon::doubleRect lReturnRect = NOPage_getBoundingBox(&mHandler,pId);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lReturnRect;
}

void NOPage::moveObject(UInt32 pId, double pDeltaX, double pDeltaY)
{
	NOPage_moveObject(&mHandler,pId,pDeltaX,pDeltaY);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
}

void NOPage::removeObject(UInt32 pId)
{
	NOPage_removeObject(&mHandler,pId);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
}

bool NOPage::isTransparentObject(UInt32 pId)
{
	bool lReturnValue = NOPage_isTransparentObject(&mHandler,pId);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lReturnValue;
}

bool NOPage::isRectangularObject(UInt32 pId)
{
	bool lReturnValue = NOPage_isRectangularObject(&mHandler,pId);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lReturnValue;
}

NOSelection NOPage::getCopyOfSelection()
{
	NOSelectionHandler lHandler = NOPage_getCopyOfSelection(&mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return NOSelection(lHandler);
}

NOSelection NOPage::getObjectsInParagraph(UInt32 pObjectID)
{
	NOSelectionHandler lHandler = NOPage_getObjectsInParagraph(&mHandler,pObjectID);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return NOSelection(lHandler);
}

void NOPage::setSelection(UInt32 pId, bool pIsSelected)
{
	NOPage_selectObject(&mHandler,pId,pIsSelected);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));	
}

void NOPage::swapSelection(UInt32 pId)
{
	NOPage_swapSelection(&mHandler,pId);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));	
}

void NOPage::clearSelection()
{
	NOPage_clearSelection(&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));	
}

void NOPage::selectMarquee( doubleRect pRect, bool pSwap, bool pIntersect )
{
	NOPage_selectXPSMarquee(&mHandler, pRect.getDoubleRectC(), pSwap, pIntersect);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));	
}

void NOPage::selectMarquee( const UInt32Rect &pRect,
							NCommon::UInt32 pWidth,
							NCommon::UInt32 pHeight,
							bool pSwap)
{
	NOPage_selectMarquee(&mHandler,pRect.getUInt32RectC(),pWidth,pHeight,pSwap,0);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));	
}

void NOPage::selectAll()
{
	NOPage_selectAll(&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));	
}

NOFont NOPage::addFontWithFamilyAndStyle(const UTF8Char *pFontFamilyName, const UTF8Char *pFontStyleName)
{
	NOFontHandler lFont = NOPage_addFontWithFamilyAndStyle(&mHandler, const_cast<UTF8Char *>(pFontFamilyName), const_cast<UTF8Char *>(pFontStyleName));
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
		
	return NOFont(lFont);
}

void NOPage::getTextProperties(UInt32 pTextObjectID,
					double &pX1, double &pY1,
					double &pX2, double &pY2,
					double &pX3, double &pY3,
					double &pX4, double &pY4,
					double &pSize)
{
	NOPage_getTextProperties(&mHandler,pTextObjectID, &pX1, &pY1, &pX2, &pY2, &pX3, &pY3, &pX4, &pY4, &pSize);
}

	
/*public*/ NOXCanvas NOPage::copyPageOnTop(const NOPage &pPage)
{
	NOXCanvas lHandler=NOPage_copyPageOnTop(&mHandler, (NOPageHandler) pPage.mHandler);
	
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	
	return lHandler;
}

bool NOPage::flattenTransparency()
{
	bool lRet = NOPage_flattenTransparency(&mHandler);
	if (mHandler.mError!=0)
		throw(NOException(mHandler));
	return lRet;
}
	
}
