// NOPartBased.cpp
//
// NOPartBased:	base class for all NiXPSObjects based on a part
//			
//
// (c) NiXPS NV, October 2006

#include "NOPartBased.h"
#include "NOString.h"
#include "NOException.h"
#include "NOPackage.h"

namespace NiXPSObjects
{

// doC# public NOHandler mHandler;

NOPartBased::NOPartBased()
{
	mHandler.mPackage=0;
	mHandler.mID=0;	
	mHandler.mType=NCommon::Part::Unknown;
	mHandler.mError=0;
}

// constructor: object is always linked to a specific package and a part ID
NOPartBased::NOPartBased(const NOHandler pHandler)
{
	mHandler=pHandler;
}

NOPartBased::NOPartBased(NOPackage *pPackage, const NCommon::UInt32 pID)
{
	mHandler.mPackage=pPackage->mHandler.mPackage;
	mHandler.mID=pID;	
	mHandler.mType=NOPackage_getTypeForID(&(pPackage->mHandler),pID);
	mHandler.mError=0;
}

// destructor
NOPartBased::~NOPartBased()
{
}
		
bool NOPartBased::isEmpty() const
{
	if (mHandler.mPackage == 0)
		return true;
	return false;
}

// test to find out is the part is in memory
bool NOPartBased::isInMemory() const
{
	return NOHandler_isInMemory((NOHandler *) &mHandler);
}

// get the part id
NCommon::UInt32 NOPartBased::getPartID() const
{
	return mHandler.mID;
}

// get the name of this part
NCommon::UTF8String NOPartBased::getPartName() const
{
	NOString lString;
//	NOHandler lHandler=lName.mHandler;
	NOHandler_getPartName((NOHandler *) &mHandler, &lString.mHandler);

	if (mHandler.mError!=0)
		throw(NOException(mHandler));

	return lString.getUTF8String();
}

// get the part typr
NCommon::Part::Type NOPartBased::getType() const
{
	return (NCommon::Part::Type) mHandler.mType;
}

}
